@@ -31,13 +31,11 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
-#include "cursor.h"
+#include "execcontext.h"
 #include "functions.h"
 #include "conditional.h"
-#include "bindings.h"
 #include "misc.h"
 #include "screen.h"
 #include "update.h"
@@ -85,6 +83,7 @@ static FvwmWindow *Circulate(char *action, int Direction, char **restofline)
 	{ /* override for Current [] */
 		mask.my_flags.use_circulate_hit = 1;
 		mask.my_flags.use_circulate_hit_icon = 1;
+		mask.my_flags.use_circulate_hit_shaded = 1;
 	}
 	CreateConditionMask(flags, &mask);
 	if (flags)
@@ -167,7 +166,8 @@ static FvwmWindow *Circulate(char *action, int Direction, char **restofline)
 }
 
 static void circulate_cmd(
-	F_CMD_ARGS, int circ_dir, Bool do_use_found)
+	F_CMD_ARGS, int new_context, int circ_dir, Bool do_use_found,
+	Bool do_exec_on_match)
 {
 	FvwmWindow *found;
 	char *restofline;
@@ -177,12 +177,18 @@ static void circulate_cmd(
 	{
 		*cond_rc = (found == NULL) ? COND_RC_NO_MATCH : COND_RC_OK;
 	}
-	if ((!found == !do_use_found) && restofline)
+	if ((!found == !do_exec_on_match) && restofline)
 	{
-		old_execute_function(
-			NULL, restofline,
-			(do_use_found) ? found : fw, eventp,
-			context, *Module, 0, NULL);
+		const exec_context_t *exc2;
+		exec_context_changes_t ecc;
+
+		ecc.w.fw = (do_use_found == True) ? found : NULL;
+		ecc.w.w = (found != NULL) ? FW_W(found) : None;
+		ecc.w.wcontext = new_context;
+		exc2 = exc_clone_context(
+			exc, &ecc, ECC_FW | ECC_W | ECC_WCONTEXT);
+		execute_function(cond_rc, exc2, restofline, 0);
+		exc_destroy_context(exc2);
 	}
 
 	return;
@@ -193,6 +199,7 @@ static void select_cmd(F_CMD_ARGS)
 	char *restofline;
 	char *flags;
 	WindowConditionMask mask;
+	FvwmWindow * const fw = exc->w.fw;
 
 	if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
 	{
@@ -206,6 +213,7 @@ static void select_cmd(F_CMD_ARGS)
 	DefaultConditionMask(&mask);
 	mask.my_flags.use_circulate_hit = 1;
 	mask.my_flags.use_circulate_hit_icon = 1;
+	mask.my_flags.use_circulate_hit_shaded = 1;
 	CreateConditionMask(flags, &mask);
 	if (flags)
 	{
@@ -217,9 +225,8 @@ static void select_cmd(F_CMD_ARGS)
 		{
 			*cond_rc = COND_RC_OK;
 		}
-		old_execute_function(
-			NULL, restofline, fw, eventp, C_WINDOW, *Module,
-			0, NULL);
+		execute_function_override_wcontext(
+			NULL, exc, restofline, 0, C_WINDOW);
 	}
 	else if (cond_rc != NULL)
 	{
@@ -253,8 +260,11 @@ char *CreateFlagString(char *string, char **restptr)
 		c++;
 	}
 
+
 	if (*c == '[' || *c == '(')
 	{
+		char *d;
+
 		/* Get the text between [ ] or ( ) */
 		if (*c == '[')
 		{
@@ -277,8 +287,11 @@ char *CreateFlagString(char *string, char **restptr)
 				*restptr = NULL;
 				return NULL;
 			}
-			c++;
-			length++;
+
+			/* skip quoted string */
+			d = SkipQuote(c, NULL, NULL, NULL);
+			length += d - c;
+			c = d;
 		}
 
 		/* We must allocate a new string because we null terminate the
@@ -351,165 +364,118 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 
 	while (condition)
 	{
-		if (StrEquals(condition,"AcceptsFocus"))
+		char *cond;
+		int on;
+
+		cond = condition;
+		on = 1;
+		if (*cond == '!')
 		{
-			mask->my_flags.do_accept_focus = 1;
+			on = 0;
+			cond++;
+		}
+		if (StrEquals(cond,"AcceptsFocus"))
+		{
+			mask->my_flags.do_accept_focus = on;
 			mask->my_flags.use_do_accept_focus = 1;
 		}
-		else if (StrEquals(condition,"!AcceptsFocus"))
+		else if (StrEquals(cond,"Focused"))
 		{
-			mask->my_flags.do_accept_focus = 0;
-			mask->my_flags.use_do_accept_focus = 1;
+			mask->my_flags.needs_focus =
+				(on) ? NEEDS_TRUE : NEEDS_FALSE;
 		}
-		else if (StrEquals(condition,"Focused"))
+		else if (StrEquals(cond,"HasPointer"))
 		{
-			mask->my_flags.needs_focus = NEEDS_TRUE;
+			mask->my_flags.needs_pointer =
+				(on) ? NEEDS_TRUE : NEEDS_FALSE;
 		}
-		else if (StrEquals(condition,"!Focused"))
+		else if (StrEquals(cond,"Iconic"))
 		{
-			mask->my_flags.needs_focus = NEEDS_FALSE;
-		}
-		else if (StrEquals(condition,"HasPointer"))
-		{
-			mask->my_flags.needs_pointer = NEEDS_TRUE;
-		}
-		else if (StrEquals(condition,"!HasPointer"))
-		{
-			mask->my_flags.needs_pointer = NEEDS_FALSE;
-		}
-		else if (StrEquals(condition,"Iconic"))
-		{
-			SET_ICONIFIED(mask, 1);
+			SET_ICONIFIED(mask, on);
 			SETM_ICONIFIED(mask, 1);
 		}
-		else if (StrEquals(condition,"!Iconic"))
+		else if (StrEquals(cond,"Visible"))
 		{
-			SET_ICONIFIED(mask, 0);
-			SETM_ICONIFIED(mask, 1);
-		}
-		else if (StrEquals(condition,"Visible"))
-		{
-			SET_PARTIALLY_VISIBLE(mask, 1);
+			SET_PARTIALLY_VISIBLE(mask, on);
 			SETM_PARTIALLY_VISIBLE(mask, 1);
 		}
-		else if (StrEquals(condition,"!Visible"))
+		else if (StrEquals(cond,"PlacedByButton3"))
 		{
-			SET_PARTIALLY_VISIBLE(mask, 0);
-			SETM_PARTIALLY_VISIBLE(mask, 1);
-		}
-		else if (StrEquals(condition,"PlacedByButton3"))
-		{
-			SET_PLACED_WB3(mask, 1);
+			SET_PLACED_WB3(mask, on);
 			SETM_PLACED_WB3(mask, 1);
 		}
-		else if (StrEquals(condition,"!PlacedByButton3"))
+		else if (StrEquals(cond,"Raised"))
 		{
-			SET_PLACED_WB3(mask, 0);
-			SETM_PLACED_WB3(mask, 1);
-		}
-		else if (StrEquals(condition,"Raised"))
-		{
-			SET_FULLY_VISIBLE(mask, 1);
+			SET_FULLY_VISIBLE(mask, on);
 			SETM_FULLY_VISIBLE(mask, 1);
 		}
-		else if (StrEquals(condition,"!Raised"))
+		else if (StrEquals(cond,"Sticky"))
 		{
-			SET_FULLY_VISIBLE(mask, 0);
-			SETM_FULLY_VISIBLE(mask, 1);
-		}
-		else if (StrEquals(condition,"Sticky"))
-		{
-			SET_STICKY(mask, 1);
+			SET_STICKY(mask, on);
 			SETM_STICKY(mask, 1);
 		}
-		else if (StrEquals(condition,"!Sticky"))
+		else if (StrEquals(cond,"Maximized"))
 		{
-			SET_STICKY(mask, 0);
-			SETM_STICKY(mask, 1);
-		}
-		else if (StrEquals(condition,"Maximized"))
-		{
-			SET_MAXIMIZED(mask, 1);
+			SET_MAXIMIZED(mask, on);
 			SETM_MAXIMIZED(mask, 1);
 		}
-		else if (StrEquals(condition,"!Maximized"))
+		else if (StrEquals(cond,"Shaded"))
 		{
-			SET_MAXIMIZED(mask, 0);
-			SETM_MAXIMIZED(mask, 1);
-		}
-		else if (StrEquals(condition,"Shaded"))
-		{
-			SET_SHADED(mask, 1);
+			SET_SHADED(mask, on);
 			SETM_SHADED(mask, 1);
 		}
-		else if (StrEquals(condition,"!Shaded"))
+		else if (StrEquals(cond,"Transient"))
 		{
-			SET_SHADED(mask, 0);
-			SETM_SHADED(mask, 1);
-		}
-		else if (StrEquals(condition,"Transient"))
-		{
-			SET_TRANSIENT(mask, 1);
+			SET_TRANSIENT(mask, on);
 			SETM_TRANSIENT(mask, 1);
 		}
-		else if (StrEquals(condition,"!Transient"))
+		else if (StrEquals(cond,"PlacedByFvwm"))
 		{
-			SET_TRANSIENT(mask, 0);
-			SETM_TRANSIENT(mask, 1);
-		}
-		else if (StrEquals(condition,"PlacedByFvwm"))
-		{
-			SET_PLACED_BY_FVWM(mask, 1);
+			SET_PLACED_BY_FVWM(mask, on);
 			SETM_PLACED_BY_FVWM(mask, 1);
 		}
-		else if (StrEquals(condition,"!PlacedByFvwm"))
+		else if (StrEquals(cond,"CurrentDesk"))
 		{
-			SET_PLACED_BY_FVWM(mask, 0);
-			SETM_PLACED_BY_FVWM(mask, 1);
+			mask->my_flags.needs_current_desk = on;
 		}
-		else if (StrEquals(condition,"CurrentDesk"))
+		else if (StrEquals(cond,"CurrentPage"))
 		{
-			mask->my_flags.needs_current_desk = 1;
+			mask->my_flags.needs_current_desk = on;
+			mask->my_flags.needs_current_page = on;
 		}
-		else if (StrEquals(condition,"CurrentPage"))
+		else if (StrEquals(cond,"CurrentGlobalPage"))
 		{
-			mask->my_flags.needs_current_desk = 1;
-			mask->my_flags.needs_current_page = 1;
+			mask->my_flags.needs_current_desk = on;
+			mask->my_flags.needs_current_global_page = on;
 		}
-		else if (StrEquals(condition,"CurrentGlobalPage"))
+		else if (StrEquals(cond,"CurrentPageAnyDesk") ||
+			 StrEquals(cond,"CurrentScreen"))
 		{
-			mask->my_flags.needs_current_desk = 1;
-			mask->my_flags.needs_current_global_page = 1;
+			mask->my_flags.needs_current_page = on;
 		}
-		else if (StrEquals(condition,"CurrentPageAnyDesk") ||
-			 StrEquals(condition,"CurrentScreen"))
+		else if (StrEquals(cond,"CurrentGlobbalPageAnyDesk"))
 		{
-			mask->my_flags.needs_current_page = 1;
+			mask->my_flags.needs_current_global_page = on;
 		}
-		else if (StrEquals(condition,"CurrentGlobbalPageAnyDesk"))
+		else if (StrEquals(cond,"CirculateHit"))
 		{
-			mask->my_flags.needs_current_global_page = 1;
+			mask->my_flags.use_circulate_hit = on;
 		}
-		else if (StrEquals(condition,"CirculateHit"))
+		else if (StrEquals(cond,"CirculateHitIcon"))
 		{
-			mask->my_flags.use_circulate_hit = 1;
+			mask->my_flags.use_circulate_hit_icon = on;
 		}
-		else if (StrEquals(condition,"CirculateHitIcon"))
+		else if (StrEquals(cond,"CirculateHitShaded"))
 		{
-			mask->my_flags.use_circulate_hit_icon = 1;
+			mask->my_flags.use_circulate_hit_shaded = on;
 		}
-		else if (StrEquals(condition,"CirculateHitShaded"))
-		{
-			mask->my_flags.use_circulate_hit_shaded = 1;
-		}
-		else if (StrEquals(condition,"State") ||
-			 StrEquals(condition,"!State"))
+		else if (StrEquals(cond,"State"))
 		{
 			if (sscanf(tmp, "%d", &state) &&
 			    state >= 0 && state <= 31)
 			{
 				state = (1 << state);
-				if (StrEquals(condition,"State"))
+				if (on)
 				{
 					SET_USER_STATES(mask, state);
 				}
@@ -523,7 +489,11 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 		}
 		else if (StrEquals(condition, "Layer"))
 		{
-			if (sscanf(tmp,"%d",&mask->layer))
+			if (on == 0)
+			{
+				mask->layer = -2;
+			}
+			else if (sscanf(tmp, "%d", &mask->layer))
 			{
 				tmp = GetNextToken (tmp, &condition);
 				if (mask->layer < 0)
@@ -544,7 +514,7 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
 			/* only 1st name to avoid mem leak */
 			mask->name = condition;
 			condition = NULL;
-			if (mask->name[0] == '!')
+			if (on == 0)
 			{
 				mask->my_flags.needs_not_name = 1;
 				mask->name++;
@@ -736,47 +706,47 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 
 void CMD_Prev(F_CMD_ARGS)
 {
-	circulate_cmd(
-		cond_rc, eventp, w, NULL, C_WINDOW, action, Module, -1, True);
+	circulate_cmd(F_PASS_ARGS, C_WINDOW, -1, True, True);
 
 	return;
 }
 
 void CMD_Next(F_CMD_ARGS)
 {
-	circulate_cmd(
-		cond_rc, eventp, w, NULL, C_WINDOW, action, Module, 1, True);
+	circulate_cmd(F_PASS_ARGS, C_WINDOW, 1, True, True);
 
 	return;
 }
 
 void CMD_None(F_CMD_ARGS)
 {
-	circulate_cmd(
-		cond_rc, eventp, w, NULL, C_ROOT, action, Module, 1, False);
+	circulate_cmd(F_PASS_ARGS, C_ROOT, 1, False, False);
 
 	return;
 }
 
 void CMD_Any(F_CMD_ARGS)
 {
-	circulate_cmd(F_PASS_ARGS, 1, False);
+	circulate_cmd(F_PASS_ARGS, exc->w.wcontext, 1, False, True);
 
 	return;
 }
 
 void CMD_Current(F_CMD_ARGS)
 {
-	circulate_cmd(
-		cond_rc, eventp, w, NULL, C_WINDOW, action, Module, 0, True);
+	circulate_cmd(F_PASS_ARGS, C_WINDOW, 0, True, True);
 
 	return;
 }
 
 void CMD_PointerWindow(F_CMD_ARGS)
 {
-	fw = get_pointer_fvwm_window();
+	exec_context_changes_t ecc;
+
+	ecc.w.fw = get_pointer_fvwm_window();
+	exc = exc_clone_context(exc, &ecc, ECC_FW);
 	select_cmd(F_PASS_ARGS);
+	exc_destroy_context(exc);
 
 	return;
 }
@@ -790,8 +760,6 @@ void CMD_ThisWindow(F_CMD_ARGS)
 
 void CMD_Pick(F_CMD_ARGS)
 {
-	DeferExecution(
-		eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease);
 	select_cmd(F_PASS_ARGS);
 
 	return;
@@ -810,6 +778,7 @@ void CMD_All(F_CMD_ARGS)
 	DefaultConditionMask(&mask);
 	mask.my_flags.use_circulate_hit = 1;
 	mask.my_flags.use_circulate_hit_icon = 1;
+	mask.my_flags.use_circulate_hit_shaded = 1;
 	CreateConditionMask(flags, &mask);
 	if (flags)
 	{
@@ -821,7 +790,7 @@ void CMD_All(F_CMD_ARGS)
 	{
 		num++;
 	}
-	g = (FvwmWindow **) safemalloc (num * sizeof(FvwmWindow *));
+	g = (FvwmWindow **)safemalloc(num * sizeof(FvwmWindow *));
 	num = 0;
 	for (t = Scr.FvwmRoot.next; t; t = t->next)
 	{
@@ -833,9 +802,8 @@ void CMD_All(F_CMD_ARGS)
 	}
 	for (i = 0; i < num; i++)
 	{
-		old_execute_function(
-			NULL, restofline, g[i], eventp, C_WINDOW, *Module, 0,
-			NULL);
+		execute_function_override_window(
+			NULL, exc, restofline, 0, g[i]);
 	}
 	if (cond_rc != NULL)
 	{
@@ -876,6 +844,7 @@ void CMD_Direction(F_CMD_ARGS)
 	float ty;
 	WindowConditionMask mask;
 	Bool is_pointer_relative;
+	FvwmWindow * const fw = exc->w.fw;
 
 	/* Parse the direction. */
 	tmp = PeekToken(action, &action);
@@ -931,7 +900,7 @@ void CMD_Direction(F_CMD_ARGS)
 	}
 	else
 	{
-		if (XQueryPointer(
+		if (FQueryPointer(
 			    dpy, Scr.Root, &JunkRoot, &JunkChild, &my_g.x,
 			    &my_g.y, &JunkX, &JunkY, &JunkMask) == False)
 		{
@@ -1032,9 +1001,8 @@ void CMD_Direction(F_CMD_ARGS)
 		{
 			*cond_rc = COND_RC_OK;
 		}
-		old_execute_function(
-			NULL, restofline, best_window, eventp, C_WINDOW,
-			*Module, 0, NULL);
+		execute_function_override_window(
+			NULL, exc, restofline, 0, best_window);
 	}
 	else if (cond_rc != NULL)
 	{
@@ -1113,6 +1081,7 @@ void CMD_WindowId(F_CMD_ARGS)
 			/* override for Current [] */
 			mask.my_flags.use_circulate_hit = 1;
 			mask.my_flags.use_circulate_hit_icon = 1;
+			mask.my_flags.use_circulate_hit_icon = 1;
 
 			CreateConditionMask(flags, &mask);
 			free(flags);
@@ -1135,9 +1104,8 @@ void CMD_WindowId(F_CMD_ARGS)
 				{
 					*cond_rc = COND_RC_OK;
 				}
-				old_execute_function(
-					NULL, action, t, eventp, C_WINDOW,
-					*Module, 0, NULL);
+				execute_function_override_window(
+					NULL, exc, action, 0, t);
 			}
 			else if (cond_rc != NULL)
 			{
@@ -1168,17 +1136,18 @@ void CMD_WindowId(F_CMD_ARGS)
 			}
 			if (action != NULL)
 			{
-				exec_func_args_type efa;
+				const exec_context_t *exc2;
+				exec_context_changes_t ecc;
 
-				memset(&efa, 0, sizeof(efa));
-				efa.cond_rc = NULL;
-				efa.win = win;
-				efa.eventp = eventp;
-				efa.action = action;
-				efa.context = C_UNMANAGED;
-				efa.module = *Module;
-				efa.flags.is_window_unmanaged = 1;
-				execute_function(&efa);
+				ecc.w.fw = NULL;
+				ecc.w.w = win;
+				ecc.w.wcontext = C_UNMANAGED;
+				exc2 = exc_clone_context(
+					exc, &ecc,
+					ECC_FW | ECC_W | ECC_WCONTEXT);
+				execute_function(
+					NULL, exc2, action, FUNC_IS_UNMANAGED);
+				exc_destroy_context(exc2);
 			}
 		}
 		else
@@ -1237,9 +1206,7 @@ void CMD_Cond(F_CMD_ARGS)
 	{
 		/* execute the command in root window context; overwrite the
 		 * return code with the return code of the command */
-		old_execute_function(
-			cond_rc, restofline, fw, eventp, context, *Module,
-			0, NULL);
+		execute_function(cond_rc, exc, restofline, 0);
 	}
 	if (flags != NULL)
 	{
@@ -1254,8 +1221,11 @@ void CMD_CondCase(F_CMD_ARGS)
 	fvwm_cond_func_rc tmp_rc;
 
 	/* same as Cond, but does not modify the return code */
-	tmp_rc = (cond_rc != NULL) ? *cond_rc : COND_RC_OK;
-	CMD_Cond(&tmp_rc, eventp, w, fw, context, action, Module);
+	if (cond_rc == NULL)
+	{
+		cond_rc = &tmp_rc;
+	}
+	CMD_Cond(F_PASS_ARGS);
 
 	return;
 }
@@ -1269,3 +1239,8 @@ void CMD_Break(F_CMD_ARGS)
 
 	return;
 }
+
+void CMD_NoWindow(F_CMD_ARGS)
+{
+	execute_function_override_window(NULL, exc, action, 0, NULL);
+}
