@@ -29,23 +29,20 @@
 
 #include <stdio.h>
 
-#include <X11/Xlib.h>
-#include <X11/Xmd.h>
-
 #include "libs/fvwmlib.h"
 #include "fvwm.h"
-#include "cursor.h"
-#include "functions.h"
+#include "execcontext.h"
+#include "commands.h"
 #include "misc.h"
 #include "screen.h"
 #include "update.h"
-#include "move_resize.h"
 #include "stack.h"
 #include "style.h"
 #include "externs.h"
 #include "decorations.h"
 #include "ewmh.h"
 #include "ewmh_intern.h"
+#include "geometry.h"
 
 typedef struct kst_item
 {
@@ -103,7 +100,7 @@ ewmh_atom ewmh_atom_client_win[] =
 
 /* EWMH_ATOM_LIST_WM_STATE
  * the different wm state, a client can ask to add, remove or toggle
- * via a _NET_WM_STATE Client message. fvwm2 must maintain these states
+ * via a _NET_WM_STATE Client message. fvwm must maintain these states
  * too */
 ewmh_atom ewmh_atom_wm_state[] =
 {
@@ -168,8 +165,8 @@ ewmh_atom ewmh_atom_property_notify[] =
 {
   ENTRY("_NET_WM_ICON",          XA_CARDINAL, ewmh_WMIcon),
   ENTRY("_NET_WM_ICON_GEOMETRY", XA_CARDINAL, ewmh_WMIconGeometry),
-  ENTRY("_NET_WM_ICON_NAME",     None,        EWMH_WMIconName_func),
-  ENTRY("_NET_WM_NAME",          None,        EWMH_WMName_func),
+  ENTRY("_NET_WM_ICON_NAME",     None,        EWMH_WMIconName),
+  ENTRY("_NET_WM_NAME",          None,        EWMH_WMName),
   ENTRY("_NET_WM_STRUT",         XA_CARDINAL, ewmh_WMStrut),
   {NULL,0,0,0}
 };
@@ -308,9 +305,9 @@ void ewmh_DeleteProperty(Window w,
 static
 void *atom_get(Window win, Atom to_get, Atom type, unsigned int *size)
 {
-  unsigned char	*retval;
-  Atom	type_ret;
-  unsigned long	 bytes_after, num_ret;
+  unsigned char *retval;
+  Atom  type_ret;
+  unsigned long  bytes_after, num_ret;
   long length;
   int  format_ret;
   void *data;
@@ -356,7 +353,7 @@ void *ewmh_AtomGetByName(Window win, const char *atom_name,
 }
 
 /* ************************************************************************* *
- *  client_root: here the client is fvwm2
+ *  client_root: here the client is fvwm
  * ************************************************************************* */
 static
 int check_desk(void)
@@ -421,7 +418,7 @@ void EWMH_SetNumberOfDesktops(void)
 }
 
 /* ************************************************************************* *
- *  client_win: here the client is fvwm2
+ *  client_win: here the client is fvwm
  * ************************************************************************* */
 void EWMH_SetActiveWindow(Window w)
 {
@@ -442,12 +439,12 @@ void EWMH_SetWMDesktop(FvwmWindow *fwin)
     ewmhc.NeedsToCheckDesk = True;
     EWMH_SetNumberOfDesktops();
   }
-  ewmh_ChangeProperty(fwin->w, "_NET_WM_DESKTOP", EWMH_ATOM_LIST_CLIENT_WIN,
+  ewmh_ChangeProperty(FW_W(fwin), "_NET_WM_DESKTOP", EWMH_ATOM_LIST_CLIENT_WIN,
 		      (unsigned char *)&desk, 1);
 }
 
 /* ************************************************************************* *
- *  fvwm2 must maintain the _NET_WM_STATE
+ *  fvwm must maintain the _NET_WM_STATE
  * ************************************************************************* */
 
 void EWMH_SetWMState(FvwmWindow *fwin, Bool do_restore)
@@ -464,10 +461,10 @@ void EWMH_SetWMState(FvwmWindow *fwin, Bool do_restore)
   }
 
   if (i > 0)
-    ewmh_ChangeProperty(fwin->w, "_NET_WM_STATE", EWMH_ATOM_LIST_CLIENT_WIN,
+    ewmh_ChangeProperty(FW_W(fwin), "_NET_WM_STATE", EWMH_ATOM_LIST_CLIENT_WIN,
 			(unsigned char *)wm_state, i);
   else
-    ewmh_DeleteProperty(fwin->w, "_NET_WM_STATE", EWMH_ATOM_LIST_CLIENT_WIN);
+    ewmh_DeleteProperty(FW_W(fwin), "_NET_WM_STATE", EWMH_ATOM_LIST_CLIENT_WIN);
 }
 
 /* ************************************************************************* *
@@ -561,7 +558,7 @@ void ewmh_AddToKdeSysTray(FvwmWindow *fwin)
   Atom *val;
   KstItem *t;
 
-  val = ewmh_AtomGetByName(fwin->w, "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
+  val = ewmh_AtomGetByName(FW_W(fwin), "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
 			   EWMH_ATOM_LIST_FIXED_PROPERTY, &size);
 
   if (val == NULL)
@@ -569,13 +566,13 @@ void ewmh_AddToKdeSysTray(FvwmWindow *fwin)
   free(val);
 
   t = ewmh_KstWinList;
-  while(t != NULL && t->w != fwin->w)
+  while(t != NULL && t->w != FW_W(fwin))
     t = t->next;
 
   if (t != NULL)
     return; /* already in the list */
 
-  add_kst_item(fwin->w);
+  add_kst_item(FW_W(fwin));
   set_kde_sys_tray();
 }
 
@@ -630,14 +627,14 @@ void EWMH_ManageKdeSysTray(Window w, int type)
     fprintf(stderr,"KST_UNMAP: 0x%lx\n", w);
 #endif
     XSelectInput(dpy, w, StructureNotifyMask);
-    XSync(dpy, 0);
+    XFlush(dpy);
     break;
   case DestroyNotify:
 #ifdef DEBUG_KST
     fprintf(stderr,"KST_DESTROY: 0x%lx\n", w);
 #endif
     XSelectInput(dpy, t->w, NoEventMask);
-    XSync(dpy, 0);
+    XFlush(dpy);
     delete_kst_item(w);
     set_kde_sys_tray();
     break;
@@ -646,7 +643,7 @@ void EWMH_ManageKdeSysTray(Window w, int type)
     fprintf(stderr,"KST_Reparent: 0x%lx\n", w);
 #endif
     XSelectInput(dpy, w, StructureNotifyMask);
-    XSync(dpy, 0);
+    XFlush(dpy);
     break;
   default:
 #ifdef DEBUG_KST
@@ -671,7 +668,7 @@ void EWMH_SetClientList()
   {
     wl = (Window *)safemalloc(sizeof (Window) * nbr);
     for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
-      wl[i++] = t->w;
+      wl[i++] = FW_W(t);
   }
   ewmh_ChangeProperty(Scr.Root,"_NET_CLIENT_LIST", EWMH_ATOM_LIST_FVWM_ROOT,
 		      (unsigned char *)wl, nbr);
@@ -693,7 +690,7 @@ void EWMH_SetClientListStacking()
   {
     wl = (Window *)safemalloc(sizeof (Window) * nbr);
     for (t = Scr.FvwmRoot.stack_next; t != &Scr.FvwmRoot; t = t->stack_next)
-      wl[i--] = t->w;
+      wl[i--] = FW_W(t);
   }
   ewmh_ChangeProperty(Scr.Root,"_NET_CLIENT_LIST_STACKING",
 		      EWMH_ATOM_LIST_FVWM_ROOT, (unsigned char *)wl, nbr);
@@ -912,20 +909,20 @@ float EWMH_GetStrutIntersection(
 void EWMH_SetFrameStrut(FvwmWindow *fwin)
 {
   CARD32 val[4];
-  int border_width = fwin->boundary_width;
+  size_borders b;
+
+  get_window_borders(fwin, &b);
 
   /* left */
-  val[0] = border_width;
+  val[0] = b.top_left.width;
   /* right */
-  val[1] = border_width;
+  val[1] = b.bottom_right.width;
   /* top */
-  val[2] = border_width +
-    ((HAS_TITLE(fwin) && !HAS_BOTTOM_TITLE(fwin))? fwin->title_g.height : 0);
+  val[2] = b.top_left.height;
   /* bottom */
-  val[3] = border_width +
-    ((HAS_TITLE(fwin) && HAS_BOTTOM_TITLE(fwin))? fwin->title_g.height : 0);
+  val[3] = b.top_left.height;
 
-  ewmh_ChangeProperty(fwin->w, "_KDE_NET_WM_FRAME_STRUT",
+  ewmh_ChangeProperty(FW_W(fwin), "_KDE_NET_WM_FRAME_STRUT",
 		      EWMH_ATOM_LIST_FVWM_WIN, (unsigned char *)&val, 4);
 }
 
@@ -971,10 +968,10 @@ void EWMH_SetAllowedActions(FvwmWindow *fwin)
   }
 
   if (i > 0)
-    ewmh_ChangeProperty(fwin->w, "_NET_WM_ALLOWED_ACTIONS",
+    ewmh_ChangeProperty(FW_W(fwin), "_NET_WM_ALLOWED_ACTIONS",
 		 EWMH_ATOM_LIST_FVWM_WIN, (unsigned char *)wm_actions, i);
   else
-    ewmh_DeleteProperty(fwin->w, "_NET_WM_ALLOWED_ACTIONS",
+    ewmh_DeleteProperty(FW_W(fwin), "_NET_WM_ALLOWED_ACTIONS",
 		 EWMH_ATOM_LIST_FVWM_WIN);
 }
 
@@ -984,11 +981,12 @@ void EWMH_SetAllowedActions(FvwmWindow *fwin)
 
 int ewmh_HandleDesktop(EWMH_CMD_ARGS)
 {
-  if (Scr.EwmhDesktop != NULL && Scr.EwmhDesktop->w != fwin->w)
+  if (Scr.EwmhDesktop != NULL && FW_W(Scr.EwmhDesktop) != FW_W(fwin))
   {
     fvwm_msg(WARN,"ewmh_HandleDesktop",
-	"An Desktop application (0x%lx) already run! This can cause problem\n",
-	 Scr.EwmhDesktop->w);
+	     "An Desktop application (0x%lx) already runs! This can cause"
+	     " problems\n",
+	     FW_W(Scr.EwmhDesktop));
     /* what to do ? */
   }
 
@@ -999,21 +997,21 @@ int ewmh_HandleDesktop(EWMH_CMD_ARGS)
   style->flags.use_layer = 1;
   style->flag_mask.use_layer = 1;
 
-  SFSET_IS_STICKY(*style, 1);
-  SMSET_IS_STICKY(*style, 1);
-  SCSET_IS_STICKY(*style, 1);
+  S_SET_IS_STICKY(SCF(*style), 1);
+  S_SET_IS_STICKY(SCM(*style), 1);
+  S_SET_IS_STICKY(SCC(*style), 1);
 
-  SFSET_IS_FIXED(*style, 1);
-  SMSET_IS_FIXED(*style, 1);
-  SCSET_IS_FIXED(*style, 1);
+  S_SET_IS_FIXED(SCF(*style), 1);
+  S_SET_IS_FIXED(SCM(*style), 1);
+  S_SET_IS_FIXED(SCC(*style), 1);
 
-  SFSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
 
-  SFSET_DO_CIRCULATE_SKIP(*style, 1);
-  SMSET_DO_CIRCULATE_SKIP(*style, 1);
-  SCSET_DO_CIRCULATE_SKIP(*style, 1);
+  S_SET_DO_CIRCULATE_SKIP(SCF(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCM(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCC(*style), 1);
 
   /* No border */
   SSET_BORDER_WIDTH(*style, 0);
@@ -1032,22 +1030,34 @@ int ewmh_HandleDesktop(EWMH_CMD_ARGS)
   style->change_mask.has_no_title = 1;
 
   /* ClickToFocus, I do not think we should use NeverFocus */
-  SFSET_FOCUS_MODE(*style, FOCUS_CLICK);
-  SMSET_FOCUS_MODE(*style, FOCUS_MASK);
-  SCSET_FOCUS_MODE(*style, FOCUS_MASK);
-  SFSET_DO_GRAB_FOCUS_WHEN_CREATED(*style, 1);
-  SMSET_DO_GRAB_FOCUS_WHEN_CREATED(*style, 1);
-  SCSET_DO_GRAB_FOCUS_WHEN_CREATED(*style, 1);
+  FPS_FOCUS_CLICK_CLIENT(S_FOCUS_POLICY(SCF(*style)), 1);
+  FPS_FOCUS_CLICK_CLIENT(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_FOCUS_CLICK_CLIENT(S_FOCUS_POLICY(SCC(*style)), 1);
+  FPS_GRAB_FOCUS(S_FOCUS_POLICY(SCF(*style)), 1);
+  FPS_GRAB_FOCUS(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_GRAB_FOCUS(S_FOCUS_POLICY(SCC(*style)), 1);
 
   /* ClickToFocusPassesClick */
-  SFSET_DO_NOT_PASS_CLICK_FOCUS_CLICK(*style, 0);
-  SMSET_DO_NOT_PASS_CLICK_FOCUS_CLICK(*style, 1);
-  SCSET_DO_NOT_PASS_CLICK_FOCUS_CLICK(*style, 1);
+  FPS_PASS_FOCUS_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+  FPS_PASS_FOCUS_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_PASS_FOCUS_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
+  FPS_PASS_RAISE_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+  FPS_PASS_RAISE_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_PASS_RAISE_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
 
   /* not useful */
-  SFSET_DO_NOT_RAISE_CLICK_FOCUS_CLICK(*style, 1);
-  SMSET_DO_NOT_RAISE_CLICK_FOCUS_CLICK(*style, 1);
-  SCSET_DO_NOT_RAISE_CLICK_FOCUS_CLICK(*style, 1);
+  FPS_RAISE_FOCUSED_CLIENT_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+  FPS_RAISE_FOCUSED_CLIENT_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_RAISE_FOCUSED_CLIENT_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
+  FPS_RAISE_UNFOCUSED_CLIENT_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+  FPS_RAISE_UNFOCUSED_CLIENT_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_RAISE_UNFOCUSED_CLIENT_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
+  FPS_RAISE_FOCUSED_DECOR_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+  FPS_RAISE_FOCUSED_DECOR_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_RAISE_FOCUSED_DECOR_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
+  FPS_RAISE_UNFOCUSED_DECOR_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+  FPS_RAISE_UNFOCUSED_DECOR_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
+  FPS_RAISE_UNFOCUSED_DECOR_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
 
   return 1;
 }
@@ -1062,17 +1072,17 @@ int ewmh_HandleDock(EWMH_CMD_ARGS)
 {
   fwin->ewmh_window_type = EWMH_WINDOW_TYPE_DOCK_ID;
 
-  SFSET_IS_STICKY(*style, 1);
-  SMSET_IS_STICKY(*style, 1);
-  SCSET_IS_STICKY(*style, 1);
+  S_SET_IS_STICKY(SCF(*style), 1);
+  S_SET_IS_STICKY(SCM(*style), 1);
+  S_SET_IS_STICKY(SCC(*style), 1);
 
-  SFSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
 
-  SFSET_DO_CIRCULATE_SKIP(*style, 1);
-  SMSET_DO_CIRCULATE_SKIP(*style, 1);
-  SCSET_DO_CIRCULATE_SKIP(*style, 1);
+  S_SET_DO_CIRCULATE_SKIP(SCF(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCM(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCC(*style), 1);
 
   /* no title ? MWM hints should be used by the app but ... */
 
@@ -1083,17 +1093,17 @@ int ewmh_HandleMenu(EWMH_CMD_ARGS)
 {
   fwin->ewmh_window_type = EWMH_WINDOW_TYPE_MENU_ID;
 
-  SFSET_IS_STICKY(*style, 1);
-  SMSET_IS_STICKY(*style, 1);
-  SCSET_IS_STICKY(*style, 1);
+  S_SET_IS_STICKY(SCF(*style), 1);
+  S_SET_IS_STICKY(SCM(*style), 1);
+  S_SET_IS_STICKY(SCC(*style), 1);
 
-  SFSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
 
-  SFSET_DO_CIRCULATE_SKIP(*style, 1);
-  SMSET_DO_CIRCULATE_SKIP(*style, 1);
-  SCSET_DO_CIRCULATE_SKIP(*style, 1);
+  S_SET_DO_CIRCULATE_SKIP(SCF(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCM(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCC(*style), 1);
 
   /* no title */
   style->flags.has_no_title = 1;
@@ -1123,17 +1133,17 @@ int ewmh_HandleToolBar(EWMH_CMD_ARGS)
 {
   fwin->ewmh_window_type = EWMH_WINDOW_TYPE_TOOLBAR_ID;
 
-  SFSET_IS_STICKY(*style, 1);
-  SMSET_IS_STICKY(*style, 1);
-  SCSET_IS_STICKY(*style, 1);
+  S_SET_IS_STICKY(SCF(*style), 1);
+  S_SET_IS_STICKY(SCM(*style), 1);
+  S_SET_IS_STICKY(SCC(*style), 1);
 
-  SFSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-  SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+  S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
 
-  SFSET_DO_CIRCULATE_SKIP(*style, 1);
-  SMSET_DO_CIRCULATE_SKIP(*style, 1);
-  SCSET_DO_CIRCULATE_SKIP(*style, 1);
+  S_SET_DO_CIRCULATE_SKIP(SCF(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCM(*style), 1);
+  S_SET_DO_CIRCULATE_SKIP(SCC(*style), 1);
 
   /* no title ? MWM hints should be used by the app but ... */
 
@@ -1147,7 +1157,7 @@ void ewmh_HandleWindowType(FvwmWindow *fwin, window_style *style)
   unsigned int size = 0;
 
   fwin->ewmh_window_type = 0;
-  val = ewmh_AtomGetByName(fwin->w, "_NET_WM_WINDOW_TYPE",
+  val = ewmh_AtomGetByName(FW_W(fwin), "_NET_WM_WINDOW_TYPE",
 			   EWMH_ATOM_LIST_FIXED_PROPERTY, &size);
 
   if (val == NULL)
@@ -1173,11 +1183,11 @@ static
 int ksmserver_workarround(FvwmWindow *fwin)
 {
 
-  if (fwin->name != NULL && fwin->class.res_name != NULL &&
-      fwin->icon_name != NULL && fwin->class.res_class != NULL &&
-      strcmp(fwin->name,"ksmserver") == 0 &&
+  if (fwin->name.name != NULL && fwin->class.res_name != NULL &&
+      fwin->icon_name.name != NULL && fwin->class.res_class != NULL &&
+      strcmp(fwin->name.name,"ksmserver") == 0 &&
       strcmp(fwin->class.res_class,"ksmserver") == 0 &&
-      strcmp(fwin->icon_name,"ksmserver") == 0 &&
+      strcmp(fwin->icon_name.name,"ksmserver") == 0 &&
       strcmp(fwin->class.res_name,"unnamed") == 0)
   {
     int layer = 0;
@@ -1207,7 +1217,7 @@ void EWMH_GetStyle(FvwmWindow *fwin, window_style *style)
 /* see also EWMH_WMName and EWMH_WMIconName in add_window */
 void EWMH_WindowInit(FvwmWindow *fwin)
 {
-  /*EWMH_DLOG("Init window 0x%lx",fwin->w);*/
+  /*EWMH_DLOG("Init window 0x%lx",FW_W(fwin));*/
   EWMH_SetWMState(fwin, False);
   EWMH_SetWMDesktop(fwin);
   EWMH_SetFrameStrut(fwin);
@@ -1215,27 +1225,47 @@ void EWMH_WindowInit(FvwmWindow *fwin)
   ewmh_WMStrut(fwin, NULL, NULL, 0);
   ewmh_WMIconGeometry(fwin, NULL, NULL, 0);
   ewmh_AddToKdeSysTray(fwin);
-  if (IS_EWMH_DESKTOP(fwin->w))
+  if (IS_EWMH_DESKTOP(FW_W(fwin)))
     return;
   if (ksmserver_workarround(fwin))
     return;
   ewmh_WMIcon(fwin, NULL, NULL, 0);
-  /*EWMH_DLOG("window 0x%lx initialised",fwin->w);*/
+  /*EWMH_DLOG("window 0x%lx initialised",FW_W(fwin));*/
 }
 
-/* a window are going to be destroyed */
+/* unmap or reparent: restore state */
+void EWMH_RestoreInitialStates(FvwmWindow *fwin, int event_type)
+{
+  EWMH_SetWMState(fwin, True);
+  if (HAS_EWMH_INIT_WM_DESKTOP(fwin) == EWMH_STATE_HAS_HINT)
+  {
+    ewmh_ChangeProperty(FW_W(fwin), "_NET_WM_DESKTOP",
+			EWMH_ATOM_LIST_CLIENT_WIN,
+			(unsigned char *)&(fwin->ewmh_hint_desktop), 1);
+  }
+  else
+  {
+    ewmh_DeleteProperty(FW_W(fwin), "_NET_WM_DESKTOP", EWMH_ATOM_LIST_CLIENT_WIN);
+  }
+  if (HAS_EWMH_WM_ICON_HINT(fwin) == EWMH_FVWM_ICON)
+      EWMH_DeleteWmIcon(fwin, True, True);
+}
+
+/* a window are going to be destroyed (in the add_window.c destroy_window
+ * sens) */
 void EWMH_DestroyWindow(FvwmWindow *fwin)
 {
-  if (IS_EWMH_DESKTOP(fwin->w))
+  if (IS_EWMH_DESKTOP(FW_W(fwin)))
     Scr.EwmhDesktop = NULL;
   if (fwin->Desk >= ewmhc.NumberOfDesktops)
     ewmhc.NeedsToCheckDesk = True;
 }
 
-/* a window has been destroyed */
+/* a window has been destroyed (unmap/reparent/destroy) */
 void EWMH_WindowDestroyed(void)
 {
   EWMH_SetClientList();
+  EWMH_SetClientListStacking();
   if (ewmhc.NeedsToCheckDesk)
     EWMH_SetNumberOfDesktops();
   ewmh_ComputeAndSetWorkArea();
@@ -1340,6 +1370,7 @@ void EWMH_Init(void)
   EWMH_SetNumberOfDesktops();
   EWMH_SetClientList();
   EWMH_SetClientListStacking();
+  ewmh_ComputeAndSetWorkArea();
 }
 
 /* ************************************************************************* *
@@ -1351,10 +1382,7 @@ void EWMH_ExitStuff(void)
 
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
   {
-    if (HAS_EWMH_WM_ICON_HINT(t) == EWMH_FVWM_ICON)
-      EWMH_DeleteWmIcon(t, True, True);
-    /* restore the wm_state */
-    EWMH_SetWMState(t, True);
+    EWMH_RestoreInitialStates(t, 0);
   }
 
 }
@@ -1381,7 +1409,7 @@ void EWMH_DLOG(char *msg, ...)
   time_taken = time_val - prev_time;
   prev_time = time_val;
   sprintf(buffer, "%.2d:%.2d:%.2d %6ld",
-          t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec, time_taken);
+	  t_ptr->tm_hour, t_ptr->tm_min, t_ptr->tm_sec, time_taken);
 
   fprintf(stderr,"EWMH DEBUG: ");
   va_start(args,msg);
