@@ -33,60 +33,47 @@
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
-
 #ifdef HAVE_GETPWUID
 #  include <pwd.h>
 #endif
+#if HAVE_SYS_SYSTEMINFO_H
+/* Solaris has sysinfo instead of gethostname.  */
+#include <sys/systeminfo.h>
+#endif
+
+#include <X11/Xproto.h>
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
-#include "libs/PictureBase.h"
 #include "libs/PictureUtils.h"
-#include "libs/Flocale.h"
-#include <libs/gravity.h>
 #include "libs/FRenderInit.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
 #include "functions.h"
-#include "bindings.h"
 #include "misc.h"
 #include "screen.h"
-#include "defaults.h"
 #include "builtins.h"
-#include "menus.h"
 #include "module_interface.h"
 #include "colorset.h"
+#include "events.h"
+#include "eventhandler.h"
+#include "eventmask.h"
 #include "icccm2.h"
 #include "gnome.h"
 #include "ewmh.h"
-#include "icons.h"
 #include "add_window.h"
 #include "fvwmsignal.h"
-#include "colormaps.h"
 #include "stack.h"
 #include "virtual.h"
 #include "session.h"
-#include "events.h"
 #include "read.h"
-#include "colors.h"
 #include "focus.h"
 #include "update.h"
-#include "window_flags.h"
 #include "move_resize.h"
-#include "borders.h"
 #include "frame.h"
-
-#include <X11/Xproto.h>
-#include <X11/Xatom.h>
-/* need to get prototype for XrmUniqueQuark for XUniqueContext call */
-#include <X11/Xresource.h>
-
-#if HAVE_SYS_SYSTEMINFO_H
-/* Solaris has sysinfo instead of gethostname.  */
-#include <sys/systeminfo.h>
-#endif
+#include "menus.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -524,10 +511,16 @@ void Done(int restart, char *command)
 	exitFuncName = getInitFunctionName(2);
 	if (functions_is_complex_function(exitFuncName))
 	{
+		const exec_context_t *exc;
+		exec_context_changes_t ecc;
+
 		char *action = safestrdup(
 			CatString2("Function ", exitFuncName));
-		old_execute_function(
-			NULL, action, NULL, &Event, C_ROOT, 1, 0, NULL);
+		ecc.type = EXCT_EXIT;
+		ecc.w.wcontext = C_ROOT;
+		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
+		execute_function(NULL, exc, action, 0);
+		exc_destroy_context(exc);
 		free(action);
 	}
 	if (!restart)
@@ -1181,6 +1174,7 @@ static void InitVariables(void)
 	Scr.gs.EmulateWIN = DEFAULT_EMULATE_WIN;
 	Scr.gs.use_active_down_buttons = DEFAULT_USE_ACTIVE_DOWN_BUTTONS;
 	Scr.gs.use_inactive_buttons = DEFAULT_USE_INACTIVE_BUTTONS;
+	Scr.gs.use_inactive_down_buttons = DEFAULT_USE_INACTIVE_DOWN_BUTTONS;
 	/* Not the right place for this, should only be called once
 	 * somewhere .. */
 
@@ -1189,7 +1183,7 @@ static void InitVariables(void)
 	Scr.PanFrameBottom.command = NULL;
 	Scr.PanFrameRight.command  = NULL;
 	Scr.PanFrameLeft.command   = NULL;
-	Scr.flags.is_pointer_on_this_screen = !!XQueryPointer(
+	Scr.flags.is_pointer_on_this_screen = !!FQueryPointer(
 		dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &JunkX,
 		&JunkY, &JunkMask);
 
@@ -1201,7 +1195,7 @@ static void InitVariables(void)
 
 static void usage(void)
 {
-	fprintf(stderr, "\nFVWM version %s Usage:\n\n",VERSION);
+	fprintf(stderr, "\nfvwm %s Usage:\n\n", VERSION);
 	fprintf(stderr, "  %s"
 		" [-d dpy]"
 		" [-debug]"
@@ -1228,15 +1222,15 @@ static void setVersionInfo(void)
 	int support_len;
 
 	/* Set version information string */
-	sprintf(version_str, "fvwm version %s compiled on %s at %s",
-		VERSION, __DATE__, __TIME__);
+	sprintf(version_str, "fvwm %s%s compiled on %s at %s",
+		VERSION, VERSIONINFO, __DATE__, __TIME__);
 	Fvwm_VersionInfo = safestrdup(version_str);
 
 	sprintf(license_str,
-		"fvwm comes with NO WARRANTY, to the extent permitted\n"
-		"by law.  You may redistribute copies of fvwm under\n"
-		"the terms of the GNU General Public License.  For\n"
-		"more information about these matters, see the file\n"
+		"fvwm comes with NO WARRANTY, to the extent permitted by law. "
+		"You may\nredistribute copies of fvwm under "
+		"the terms of the GNU General Public License.\n"
+		"For more information about these matters, see the file "
 		"named COPYING.");
 	Fvwm_LicenseInfo = safestrdup(license_str);
 
@@ -1267,7 +1261,7 @@ static void setVersionInfo(void)
 	strcat(support_str, " SM,");
 #endif
 #ifdef HAVE_BIDI
-	strcat(support_str, " bidi text,");
+	strcat(support_str, " Bidi text,");
 #endif
 #ifdef HAVE_XINERAMA
 	strcat(support_str, " Xinerama,");
@@ -1320,8 +1314,14 @@ static void SetRCDefaults(void)
 
 	for (i = 0; defaults[i]; i++)
 	{
-		old_execute_function(
-			NULL, defaults[i], NULL, &Event, C_ROOT, 1, 0, NULL);
+		const exec_context_t *exc;
+		exec_context_changes_t ecc;
+
+		ecc.type = EXCT_INIT;
+		ecc.w.wcontext = C_ROOT;
+		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
+		execute_function(NULL, exc, defaults[i], 0);
+		exc_destroy_context(exc);
 	}
 
 	return;
@@ -1388,8 +1388,13 @@ void StartupStuff(void)
 {
 #define startFuncName "StartFunction"
 	const char *initFuncName;
+	const exec_context_t *exc;
+	exec_context_changes_t ecc;
 
-	CaptureAllWindows(False);
+	ecc.type = EXCT_INIT;
+	ecc.w.wcontext = C_ROOT;
+	exc = exc_create_context(&ecc, ECC_TYPE | ECC_W);
+	CaptureAllWindows(exc, False);
 	/* Turn off the SM stuff after the initial capture so that new windows
 	 * will not be matched by accident. */
 	if (Restarting)
@@ -1420,8 +1425,8 @@ void StartupStuff(void)
 	if (functions_is_complex_function(startFuncName))
 	{
 		char *action = "Function " startFuncName;
-		old_execute_function(
-			NULL, action, NULL, &Event, C_ROOT, 1, 0, NULL);
+
+		execute_function(NULL, exc, action, 0);
 	}
 
 	/* migo (03-Jul-1999): execute [Session]{Init|Restart}Function */
@@ -1430,8 +1435,8 @@ void StartupStuff(void)
 	{
 		char *action = safestrdup(
 			CatString2("Function ", initFuncName));
-		old_execute_function(
-			NULL, action, NULL, &Event, C_ROOT, 1, 0, NULL);
+
+		execute_function(NULL, exc, action, 0);
 		free(action);
 	}
 
@@ -1449,6 +1454,7 @@ void StartupStuff(void)
 	{
 		unlink(state_filename);
 	}
+	exc_destroy_context(exc);
 
 	return;
 }
@@ -1593,9 +1599,12 @@ int main(int argc, char **argv)
 	Bool option_error = False;
 	int visualClass = -1;
 	int visualId = -1;
+	const exec_context_t *exc;
+	exec_context_changes_t ecc;
 
 	DBUG("main", "Entered, about to parse args");
 
+	memset(&Scr, 0, sizeof(Scr));
 	/* for use on restart */
 	g_argv = (char **)safemalloc((argc + 4) * sizeof(char *));
 	g_argc = argc;
@@ -1778,27 +1787,27 @@ int main(int argc, char **argv)
 			{
 				usage();
 			}
-			if (StrEquals(argv[i], "staticg"))
+			if (strncasecmp(argv[i], "staticg", 7) == 0)
 			{
 				visualClass = StaticGray;
 			}
-			else if (StrEquals(argv[i], "g"))
+			else if (strncasecmp(argv[i], "g", 1) == 0)
 			{
 				visualClass = GrayScale;
 			}
-			else if (StrEquals(argv[i], "staticc"))
+			else if (strncasecmp(argv[i], "staticc", 7) == 0)
 			{
 				visualClass = StaticColor;
 			}
-			else if (StrEquals(argv[i], "p"))
+			else if (strncasecmp(argv[i], "p", 1) == 0)
 			{
 				visualClass = PseudoColor;
 			}
-			else if (StrEquals(argv[i], "t"))
+			else if (strncasecmp(argv[i], "t", 1) == 0)
 			{
 				visualClass = TrueColor;
 			}
-			else if (StrEquals(argv[i], "d"))
+			else if (strncasecmp(argv[i], "d", 1) == 0)
 			{
 				visualClass = DirectColor;
 			}
@@ -1819,18 +1828,10 @@ int main(int argc, char **argv)
 		else if (StrEquals(argv[i], "-version") ||
 			 StrEquals(argv[i], "--version"))
 		{
-			printf("%s\n%s\n%s\n", Fvwm_VersionInfo,
-			       Fvwm_LicenseInfo, Fvwm_SupportInfo);
+			printf("%s\n%s\n\n%s\n", Fvwm_VersionInfo,
+			       Fvwm_SupportInfo, Fvwm_LicenseInfo);
 			exit(0);
 		}
-		else if (StrEquals(argv[i], "-color-limit"))
-		{
-			if (++i >= argc)
-			{
-				usage();
-			}
-			CopyString(&opt_color_limit,argv[i]);
-		}
 		else
 		{
 			fvwm_msg(ERR, "main", "Unknown option: `%s'", argv[i]);
@@ -1982,8 +1983,16 @@ int main(int argc, char **argv)
 			XFree(vizinfo);
 			/* have to have a colormap for non-default visual
 			 * windows */
-			Pcmap = XCreateColormap(
-				dpy, Scr.Root, Pvisual, AllocNone);
+			if (Pvisual->class == DirectColor)
+			{
+				Pcmap = XCreateColormap(
+					dpy, Scr.Root, Pvisual, AllocAll);
+			}
+			else
+			{
+				Pcmap = XCreateColormap(
+					dpy, Scr.Root, Pvisual, AllocNone);
+			}
 			Pdefault = False;
 		}
 		else
@@ -2014,8 +2023,16 @@ int main(int argc, char **argv)
 			XFree(vizinfo);
 			/* have to have a colormap for non-default visual
 			 * windows */
-			Pcmap = XCreateColormap(
-				dpy, Scr.Root, Pvisual, AllocNone);
+			if (Pvisual->class == DirectColor)
+			{
+				Pcmap = XCreateColormap(
+					dpy, Scr.Root, Pvisual, AllocAll);
+			}
+			else
+			{
+				Pcmap = XCreateColormap(
+					dpy, Scr.Root, Pvisual, AllocNone);
+			}
 			Pdefault = False;
 		}
 		else
@@ -2030,28 +2047,69 @@ int main(int argc, char **argv)
 	/* use default visuals if none found so far */
 	if (visualClass == -1 && visualId == -1)
 	{
-		Pvisual = DefaultVisual(dpy, Scr.screen);
-		Pdepth = DefaultDepth(dpy, Scr.screen);
-		Pcmap = DefaultColormap(dpy, Scr.screen);
-		Pdefault = True;
+		Colormap *cmap_list = NULL;
+		int num, i;
+
+		Pdepth = 0;
+
+		/* Detection of a card with 2 hardware colormaps (8+24) which use
+		 * depth 8 for the default. We can use our own depth 24 cmap
+		 * without affecting other applications. */
+		if (DefaultDepth(dpy, Scr.screen) <= 8 &&
+		    (cmap_list = XListInstalledColormaps(dpy, Scr.Root, &num))
+		    && num > 1)
+		{
+			XVisualInfo template, *vizinfo;
+			int total;
+
+			template.screen = Scr.screen;
+			template.class = TrueColor;
+			vizinfo = XGetVisualInfo(
+				dpy, VisualScreenMask|VisualClassMask,
+				&template, &total);
+			for(i = 0; i<total; i++)
+			{
+				if (Pdepth < vizinfo[i].depth &&
+				    vizinfo[i].depth > 8)
+				{
+					Pvisual = vizinfo[i].visual;
+					Pdepth = vizinfo[i].depth;
+				}
+			}
+			XFree(vizinfo);
+			if (Pdepth > 0)
+			{
+				Pcmap = XCreateColormap(
+					dpy, Scr.Root, Pvisual, AllocNone);
+				Pdefault = False;
+			}
+		}
+		if (cmap_list)
+		{
+			XFree(cmap_list);
+		}
+		if (Pdepth == 0)
+		{
+			Pvisual = DefaultVisual(dpy, Scr.screen);
+			Pdepth = DefaultDepth(dpy, Scr.screen);
+			Pcmap = DefaultColormap(dpy, Scr.screen);
+			Pdefault = True;
+		}
 	}
 
+	PictureSetupWhiteAndBlack();
+
 	/* make a copy of the visual stuff so that XorPixmap can swap with root
 	 */
 	PictureSaveFvwmVisual();
 
 	Scr.ColorLimit = 0;
 	PUseDynamicColors = 0;
-	/* dynamically changeable visual class are odd numbered */
-	if (Pdepth <= 8 && (Pvisual->class & 1))
+	Scr.ColorLimit = PictureInitColors(
+		PICTURE_CALLED_BY_FVWM, True, opt_color_limit, True, True);
+	if (opt_color_limit)
 	{
-		/* limit the number of colors */
-		Scr.ColorLimit = PictureAllocColorTable(
-			opt_color_limit, PICTURE_CALLED_BY_FVWM);
-		if (opt_color_limit)
-		{
-			free(opt_color_limit);
-		}
+		free(opt_color_limit);
 	}
 
 	FShapeInit(dpy);
@@ -2161,15 +2219,16 @@ int main(int argc, char **argv)
 	simplify_style_list();
 
 	DBUG("main", "Running config_commands...");
+	ecc.type = EXCT_INIT;
+	ecc.w.wcontext = C_ROOT;
+	exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
 	if (num_config_commands > 0)
 	{
 		int i;
 		for (i=0;i<num_config_commands;i++)
 		{
 			DoingCommandLine = True;
-			old_execute_function(
-				NULL, config_commands[i], NULL, &Event, C_ROOT,
-				1, 0, NULL);
+			execute_function(NULL, exc, config_commands[i], 0);
 			free(config_commands[i]);
 		}
 		DoingCommandLine = False;
@@ -2180,49 +2239,39 @@ int main(int argc, char **argv)
 		 * FVWM_DATADIR, and for compatibility: ~/.fvwm2rc,
 		 * $sysconfdir/system.fvwm2rc */
 		if (!run_command_file(
-			    CatString3(fvwm_userdir, "/", FVWMRC),
-			    &Event, NULL, C_ROOT, 1) &&
+			    CatString3(fvwm_userdir, "/", FVWMRC), exc) &&
 		    !run_command_file(
-			    CatString3(home_dir, "/", FVWMRC),
-			    &Event, NULL, C_ROOT, 1 ) &&
+			    CatString3(home_dir, "/", FVWMRC), exc) &&
 		    !run_command_file(
-			    CatString3(FVWM_DATADIR, "/", FVWMRC),
-			    &Event, NULL, C_ROOT, 1 ) &&
+			    CatString3(FVWM_DATADIR, "/", FVWMRC), exc) &&
 		    !run_command_file(
-			    CatString3(FVWM_DATADIR, "/system", FVWMRC),
-			    &Event, NULL, C_ROOT, 1 ) &&
+			    CatString3(FVWM_DATADIR, "/system", FVWMRC), exc) &&
 		    !run_command_file(
-			    CatString3(FVWM_CONFDIR, "/system", FVWMRC),
-			    &Event, NULL, C_ROOT, 1 ) )
+			    CatString3(FVWM_CONFDIR, "/system", FVWMRC), exc))
 		{
 			fvwm_msg(
 				ERR, "main", "Cannot read startup file, tried: "
 				"\n\t%s/%s\n\t%s/%s\n\t%s/%s\n\t%s/system"
-				"%s\n\t%s/system%s",
-				fvwm_userdir, FVWMRC,
-				home_dir,     FVWMRC,
-				FVWM_DATADIR, FVWMRC,
-				FVWM_DATADIR, FVWMRC,
-				FVWM_CONFDIR, FVWMRC);
+				"%s\n\t%s/system%s", fvwm_userdir, FVWMRC,
+				home_dir, FVWMRC, FVWM_DATADIR, FVWMRC,
+				FVWM_DATADIR, FVWMRC, FVWM_CONFDIR, FVWMRC);
 		}
 	}
+	exc_destroy_context(exc);
 
 	DBUG("main", "Done running config_commands");
 
 	if (Pdepth<2)
 	{
 		Scr.gray_pixmap = XCreatePixmapFromBitmapData(
-			dpy,Scr.NoFocusWin, g_bits, g_width, g_height,
-			BlackPixel(dpy, Scr.screen),
-			WhitePixel(dpy, Scr.screen), Pdepth);
+			dpy, Scr.NoFocusWin, g_bits, g_width, g_height,
+			PictureBlackPixel(), PictureWhitePixel(), Pdepth);
 		Scr.light_gray_pixmap = XCreatePixmapFromBitmapData(
 			dpy, Scr.NoFocusWin, l_g_bits, l_g_width, l_g_height,
-			BlackPixel(dpy, Scr.screen),
-			WhitePixel(dpy, Scr.screen), Pdepth);
+			PictureBlackPixel(), PictureWhitePixel(), Pdepth);
 		Scr.sticky_gray_pixmap = XCreatePixmapFromBitmapData(
 			dpy, Scr.NoFocusWin, s_g_bits, s_g_width, s_g_height,
-			BlackPixel(dpy, Scr.screen),
-			WhitePixel(dpy, Scr.screen), Pdepth);
+			PictureBlackPixel(), PictureWhitePixel(), Pdepth);
 	}
 
 	attributes.background_pixel = Scr.StdBack;
@@ -2235,22 +2284,16 @@ int main(int argc, char **argv)
 		InputOutput, Pvisual, valuemask, &attributes);
 	resize_geometry_window();
 	initPanFrames();
-
 	MyXGrabServer(dpy);
 	checkPanFrames();
 	MyXUngrabServer(dpy);
-
 	CoerceEnterNotifyOnCurrentWindow();
-
 	SessionInit();
-
 	GNOME_Init();
-
 	DBUG("main", "Entering HandleEvents loop...");
 
 	HandleEvents();
-
-	switch( fvwmRunState )
+	switch (fvwmRunState)
 	{
 	case FVWM_DONE:
 		Done(0, NULL);     /* does not return */
