@@ -30,11 +30,10 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
+#include "execcontext.h"
 #include "cursor.h"
-#include "functions.h"
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
@@ -42,12 +41,11 @@
 #include "geometry.h"
 #include "update.h"
 #include "style.h"
-#include "borders.h"
 #include "move_resize.h"
 #include "virtual.h"
 #include "stack.h"
-#include "add_window.h"
 #include "ewmh.h"
+#include "icons.h"
 
 #ifndef MIN
 #define MIN(A,B) ((A)<(B)? (A):(B))
@@ -538,7 +536,7 @@ static float test_fit(
  *
  **************************************************************************/
 Bool PlaceWindow(
-	FvwmWindow *fw, style_flags *sflags, rectangle *attr_g,
+	const exec_context_t *exc, style_flags *sflags, rectangle *attr_g,
 	int Desk, int PageX, int PageY, int XineramaScreen, int mode,
 	initial_window_options_type *win_opts)
 {
@@ -567,6 +565,7 @@ Bool PlaceWindow(
     unsigned do_not_use_wm_placement : 1;
   } flags;
   extern Bool Restarting;
+  FvwmWindow *fw = exc->w.fw;
 
   memset(&flags, 0, sizeof(flags));
 
@@ -904,8 +903,8 @@ Bool PlaceWindow(
 	  XMapRaised(dpy, Scr.SizeWindow);
 	}
 	FScreenGetScrRect(NULL, FSCREEN_GLOBAL, &mx, &my, NULL, NULL);
-	if (moveLoop(fw, mx, my, DragWidth, DragHeight,
-		     &xl, &yt, False))
+	if (__move_loop(
+		    exc, mx, my, DragWidth, DragHeight, &xl, &yt, False))
 	{
 	  /* resize too */
 	  rc = True;
@@ -1141,7 +1140,6 @@ Bool PlaceWindow(
   return rc;
 }
 
-#include "icons.h"
 void CMD_PlaceAgain(F_CMD_ARGS)
 {
 	char *token;
@@ -1152,12 +1150,8 @@ void CMD_PlaceAgain(F_CMD_ARGS)
 	XWindowAttributes attr;
 	Bool do_move_animated = False;
 	Bool do_place_icon = False;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
-	{
-		return;
-	}
 	if (!XGetWindowAttributes(dpy, FW_W(fw), &attr))
 	{
 		return;
@@ -1202,7 +1196,7 @@ void CMD_PlaceAgain(F_CMD_ARGS)
 		attr_g.width = attr.width;
 		attr_g.height = attr.height;
 		PlaceWindow(
-			fw, &style.flags, &attr_g, SGET_START_DESK(style),
+			exc, &style.flags, &attr_g, SGET_START_DESK(style),
 			SGET_START_PAGE_X(style), SGET_START_PAGE_Y(style),
 			SGET_START_SCREEN(style), PLACE_AGAIN, &win_opts);
 		AnimatedMoveFvwmWindow(
