@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -5,12 +6,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
  */
 
 #include "config.h"
@@ -43,7 +44,17 @@
 
 /* ----------------------------- stack ring code --------------------------- */
 
+/* If more than this many transients are in a single branch of a transient
+ * tree, they will end up in more or less random stacking order. */
+#define MAX_TRANSIENTS_IN_BRANCH 200000
+/* Same for total levels of transients. */
+#define MAX_TRANSIENT_LEVELS     10000
+/* This number must fit in a signed int! */
+#define LOWER_PENALTY (MAX_TRANSIENTS_IN_BRANCH * MAX_TRANSIENT_LEVELS)
+
 extern Bool is_frame_hide_window();
+static void __RaiseOrLowerWindow(
+	FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window);
 static void RaiseOrLowerWindow(
 	FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window);
 #if 0
@@ -52,6 +63,8 @@ static void ResyncFvwmStackRing(void);
 static void ResyncXStackingOrder(void);
 static void BroadcastRestack(FvwmWindow *s1, FvwmWindow *s2);
 static Bool is_above_unmanaged(FvwmWindow *fw, Window *umtop);
+static int collect_transients_recursive(
+	FvwmWindow *t, FvwmWindow *list_head, int layer, Bool do_lower);
 
 #define DEBUG_STACK_RING 1
 #ifdef DEBUG_STACK_RING
@@ -246,23 +259,25 @@ static void add_windowlist_to_stack_ring_after(
 	return;
 }
 
-FvwmWindow *get_next_window_in_stack_ring(FvwmWindow *t)
+FvwmWindow *get_next_window_in_stack_ring(const FvwmWindow *t)
 {
 	return t->stack_next;
 }
 
-FvwmWindow *get_prev_window_in_stack_ring(FvwmWindow *t)
+FvwmWindow *get_prev_window_in_stack_ring(const FvwmWindow *t)
 {
 	return t->stack_prev;
 }
 
-FvwmWindow *get_transientfor_fvwmwindow(FvwmWindow *t)
+FvwmWindow *get_transientfor_fvwmwindow(const FvwmWindow *t)
 {
 	FvwmWindow *s;
 
 	if (!t || !IS_TRANSIENT(t) || FW_W_TRANSIENTFOR(t) == Scr.Root ||
 	    FW_W_TRANSIENTFOR(t) == None)
+	{
 		return NULL;
+	}
 	for (s = Scr.FvwmRoot.next; s != NULL; s = s->next)
 	{
 		if (FW_W(s) == FW_W_TRANSIENTFOR(t))
@@ -305,7 +320,6 @@ Bool position_new_window_in_stack_ring(FvwmWindow *t, Bool do_lower)
 	return True;
 }
 
-
 /********************************************************************
  * Raise a target and all higher FVWM-managed windows above any
  *  override_redirects:
@@ -332,7 +346,7 @@ static void raise_over_unmanaged(FvwmWindow *t)
 
 	 * Count the windows we need to restack, then build the stack list.
 	 ********************************************************************/
-	if (! is_above_unmanaged(t, &OR_Above))
+	if (!is_above_unmanaged(t, &OR_Above))
 	{
 		for (count = 0, t2 = Scr.FvwmRoot.stack_next;
 		     t2 != &Scr.FvwmRoot; t2 = t2->stack_next)
@@ -386,12 +400,9 @@ static void raise_over_unmanaged(FvwmWindow *t)
 	return;
 }
 
-
 static Bool must_move_transients(
-	FvwmWindow *t, Bool do_lower, Bool *found_transient)
+	FvwmWindow *t, Bool do_lower)
 {
-	*found_transient = False;
-
 	if (IS_ICONIFIED(t))
 	{
 		return False;
@@ -403,7 +414,8 @@ static Bool must_move_transients(
 		Bool scanning_above_window = True;
 		FvwmWindow *q;
 
-		for (q = Scr.FvwmRoot.stack_next; q != &Scr.FvwmRoot;
+		for (q = Scr.FvwmRoot.stack_next;
+		     q != &Scr.FvwmRoot && t->layer <= q->layer;
 		     q = q->stack_next)
 		{
 			if (t->layer < q->layer)
@@ -411,12 +423,6 @@ static Bool must_move_transients(
 				/* We're not interested in higher layers. */
 				continue;
 			}
-			if (t->layer > q->layer)
-			{
-				/* We are at the end of this layer. Stop
-				 * scanning windows. */
-				break;
-			}
 			else if (t == q)
 			{
 				/* We found our window. All further transients
@@ -426,23 +432,12 @@ static Bool must_move_transients(
 			else if (IS_TRANSIENT(q) &&
 				 FW_W_TRANSIENTFOR(q) == FW_W(t))
 			{
-				/* found a transient */
-				*found_transient = True;
-
-				/* transients are not allowed below main in the
-				 * same layer */
-				if ( !scanning_above_window )
-				{
-					return True;
-				}
+				return True;
 			}
-			else if ((scanning_above_window && !do_lower) ||
-				 (*found_transient && do_lower))
+			else if (scanning_above_window && !do_lower)
 			{
 				/* raise: The window is not raised, so itself
 				 * and all transients will be raised. */
-				/* lower: There is a transient above some other
-				 * window, so we have to lower. */
 				return True;
 			}
 		}
@@ -450,31 +445,54 @@ static Bool must_move_transients(
 	return False;
 }
 
+static Window __get_stacking_sibling(FvwmWindow *fw, Bool do_stack_below)
+{
+	Window w;
+
+	/* default to frame window */
+	w = FW_W_FRAME(fw);
+	if (IS_ICONIFIED(fw) && do_stack_below == True)
+	{
+		/* override with icon windows when stacking below */
+		if (FW_W_ICON_PIXMAP(fw) != None)
+		{
+			w = FW_W_ICON_PIXMAP(fw);
+		}
+		else if (FW_W_ICON_TITLE(fw) != None)
+		{
+			w = FW_W_ICON_TITLE(fw);
+		}
+	}
+
+	return w;
+}
+
 static void restack_windows(
-	FvwmWindow *r, FvwmWindow *s, int count, Bool do_broadcast_all)
+	FvwmWindow *r, FvwmWindow *s, int count, Bool do_broadcast_all,
+	Bool do_lower)
 {
 	FvwmWindow *t;
 	unsigned int flags;
 	int i;
 	XWindowChanges changes;
 	Window *wins;
+	int do_stack_above;
+	int is_reversed;
 
 	if (count <= 0)
 	{
-		FvwmWindow *prev;
-
-		for (t = r, prev = NULL; prev != s; prev = t, t = t->stack_next)
+		for (count = 0, t = r->stack_next; t != s; t = t->stack_next)
 		{
 			count++;
 			count += get_visible_icon_window_count(t);
 		}
 	}
 	/* restack the windows between r and s */
-	wins = (Window*) safemalloc (count * sizeof (Window));
+	wins = (Window*) safemalloc ((count + 3) * sizeof (Window));
 	i = 0;
 	for (t = r->stack_next; t != s; t = t->stack_next)
 	{
-		if (i >= count)
+		if (i > count)
 		{
 			fvwm_msg(
 				ERR, "restack_windows",
@@ -484,34 +502,44 @@ static void restack_windows(
 		wins[i++] = FW_W_FRAME(t);
 		if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
 		{
-			if(FW_W_ICON_TITLE(t) != None)
+			if (FW_W_ICON_TITLE(t) != None)
 			{
 				wins[i++] = FW_W_ICON_TITLE(t);
 			}
-			if(FW_W_ICON_PIXMAP(t) != None)
+			if (FW_W_ICON_PIXMAP(t) != None)
 			{
 				wins[i++] = FW_W_ICON_PIXMAP(t);
 			}
 		}
 	}
-
-	changes.sibling = FW_W_FRAME(s);
-	if (changes.sibling != None)
+	changes.sibling = __get_stacking_sibling(r, True);
+	if (changes.sibling == None)
 	{
-		changes.stack_mode = Above;
-		flags = CWSibling|CWStackMode;
+		changes.sibling = __get_stacking_sibling(s, False);
+		is_reversed = 1;
 	}
 	else
 	{
-		changes.stack_mode = Below;
+		is_reversed = 0;
+	}
+	if (changes.sibling == None)
+	{
+		do_stack_above = !do_lower;
 		flags = CWStackMode;
 	}
-	XConfigureWindow (dpy, FW_W_FRAME(r->stack_next), flags, &changes);
-	XRestackWindows (dpy, wins, count);
+	else
+	{
+		do_stack_above = 0;
+		flags = CWStackMode | CWSibling;
+	}
+	changes.stack_mode = (do_stack_above ^ is_reversed) ? Above : Below;
+	XConfigureWindow(dpy, FW_W_FRAME(r->stack_next), flags, &changes);
+	if (count > 1)
+	{
+		XRestackWindows(dpy, wins, count);
+	}
 	free(wins);
-
 	EWMH_SetClientListStacking();
-
 	if (do_broadcast_all)
 	{
 		/* send out M_RESTACK for all windows, to make sure we don't
@@ -528,15 +556,190 @@ static void restack_windows(
 	return;
 }
 
-static void RaiseOrLowerWindow(
+static void __sort_transient_ring(FvwmWindow *ring)
+{
+	FvwmWindow *s;
+	FvwmWindow *t;
+	FvwmWindow *u;
+	FvwmWindow *prev;
+
+	if (ring->stack_next->stack_next == ring)
+	{
+		/* only one or zero windows */
+		return;
+	}
+	/* Implementation note:  this sorting algorithm is about the most
+	 * inefficient possible.  It just swaps the position of two adjacent
+	 * windows in the ring if they are in the wrong order.  Since
+	 * transient windows are rare, this should not cause any notable
+	 * performance hit.  Because it is important that the order of windows
+	 * with the same key is not changed, we can not just use qsort() here.
+	 */
+	for (t = ring->stack_next, prev = ring; t->stack_next != ring;
+	     prev = t->stack_prev)
+	{
+		s = t->stack_next;
+		if (t->scratch.i < s->scratch.i)
+		{
+			/* swap windows */
+			u = s->stack_next;
+			s->stack_next = t;
+			t->stack_next = u;
+			u = t->stack_prev;
+			t->stack_prev = s;
+			s->stack_prev = u;
+			s->stack_prev->stack_next = s;
+			t->stack_next->stack_prev = t;
+			if (prev != ring)
+			{
+				/* move further up the ring? */
+				t = prev;
+			}
+			else
+			{
+				/* hit start of ring */
+			}
+		}
+		else
+		{
+			/* correct order, advance one window */
+			t = t->stack_next;
+		}
+	}
+
+	return;
+}
+
+static Bool __restack_window(
+	FvwmWindow *t, Bool do_lower, Bool do_restack_transients,
+	Bool is_new_window)
+{
+	FvwmWindow *s, *r, tmp_r;
+	int count;
+	int test_layer;
+
+	/* detach t, so it doesn't make trouble in the loops */
+	remove_window_from_stack_ring(t);
+
+	count = 0;
+	if (do_restack_transients)
+	{
+		/* collect the transients in a temp list */
+		tmp_r.stack_prev = &tmp_r;
+		tmp_r.stack_next = &tmp_r;
+		count = collect_transients_recursive(
+			t, &tmp_r, t->layer, do_lower);
+		if (count == 0)
+		{
+			do_restack_transients = False;
+		}
+	}
+	count += 1 + get_visible_icon_window_count(t);
+
+	test_layer = t->layer;
+	if (do_lower)
+	{
+		test_layer--;
+	}
+	/* now find the place to reinsert t and friends */
+	for (s = Scr.FvwmRoot.stack_next; s != &Scr.FvwmRoot;
+	     s = s->stack_next)
+	{
+		if (test_layer >= s->layer)
+		{
+			break;
+		}
+	}
+	r = s->stack_prev;
+
+	if (do_restack_transients)
+	{
+		/* re-sort the transient windows according to their scratch.i
+		 * register */
+		__sort_transient_ring(&tmp_r);
+		/* insert all transients between r and s. */
+		add_windowlist_to_stack_ring_after(&tmp_r, r);
+	}
+
+	/*
+	** Re-insert t - below transients
+	*/
+	add_window_to_stack_ring_after(t, s->stack_prev);
+
+	if (is_new_window && IS_TRANSIENT(t) &&
+	    DO_STACK_TRANSIENT_PARENT(t) && !IS_ICONIFIED(t))
+	{
+		/* now that the new transient is properly positioned in the
+		 * stack ring, raise/lower it again so that its parent is
+		 * raised/lowered too */
+		RaiseOrLowerWindow(t, do_lower, True, False);
+		/* make sure the stacking order is correct - may be the
+		 * sledge-hammer method, but the recursion ist too hard to
+		 * understand. */
+		ResyncXStackingOrder();
+		return True;
+	}
+	else
+	{
+		/* restack the windows between r and s */
+		restack_windows(r, s, count, do_restack_transients, do_lower);
+	}
+
+	return False;
+}
+
+Bool __raise_lower_recursion(
+	FvwmWindow *t, Bool do_lower)
+{
+	FvwmWindow *t2;
+
+	for (t2 = Scr.FvwmRoot.stack_next; t2 != &Scr.FvwmRoot;
+	     t2 = t2->stack_next)
+	{
+		if (FW_W(t2) == FW_W_TRANSIENTFOR(t))
+		{
+			if (t2 == t)
+			{
+				return False;
+			}
+			if (IS_ICONIFIED(t2) || t->layer != t2->layer)
+			{
+				break;
+			}
+			if (do_lower &&
+			    (!IS_TRANSIENT(t2) ||
+			     !DO_STACK_TRANSIENT_PARENT(t2)))
+			{
+				/* hit the highest level transient; lower this
+				 * subtree below all other subtrees of the
+				 * same window */
+				t->scratch.i = -LOWER_PENALTY;
+			}
+			else
+			{
+				/* Add a bonus to the stack ring position for
+				 * this branch of the transient tree over all
+				 * other branches. */
+				t->scratch.i = MAX_TRANSIENTS_IN_BRANCH;
+			}
+			__RaiseOrLowerWindow(t2, do_lower, True, False);
+			if ((!do_lower && DO_RAISE_TRANSIENT(t2)) ||
+			    (do_lower && DO_LOWER_TRANSIENT(t2))  )
+			{
+				/* moving the parent moves our window already */
+				return True;
+			}
+		}
+	}
+
+	return False;
+}
+
+static void __RaiseOrLowerWindow(
 	FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window)
 {
-	FvwmWindow *s, *r, *t2, *next, tmp_r;
-	int count;
+	FvwmWindow *t2;
 	Bool do_move_transients;
-	Bool found_transient;
-	Bool no_movement;
-	int test_layer;
 
 	/* Do not raise this window after command execution (see
 	 * HandleButtonPress()). */
@@ -544,11 +747,11 @@ static void RaiseOrLowerWindow(
 
 	/* New windows are simply raised/lowered without touching the
 	 * transientfor at first.  Then, further down in the code,
-	 * RaiseOrLowerWindow() is called again to raise/lower the transientfor
-	 * if necessary.  We can not do the recursion stuff for new windows
-	 * because the must_move_transients() call needs a properly ordered
-	 * stack ring - but the new window is still at the front of the stack
-	 * ring. */
+	 * __RaiseOrLowerWindow() is called again to raise/lower the
+	 * transientfor if necessary.  We can not do the recursion stuff for
+	 * new windows because the must_move_transients() call needs a
+	 * properly ordered stack ring - but the new window is still at the
+	 * front of the stack ring. */
 	if (allow_recursion && !is_new_window && !IS_ICONIFIED(t))
 	{
 		/* This part makes Raise/Lower on a Transient act on its Main
@@ -558,147 +761,28 @@ static void RaiseOrLowerWindow(
 		 * most cases. This code does not handle the case where there
 		 * are trees of Main + Transient (i.e. where a
 		 * Main_window_with_Transients is itself Transient for another
-		 * window).
-		 *
-		 * Another strategy is required to handle trees of
-		 * Main+Transients */
+		 * window). */
 		if (IS_TRANSIENT(t) && DO_STACK_TRANSIENT_PARENT(t))
 		{
-			for (t2 = Scr.FvwmRoot.stack_next; t2 != &Scr.FvwmRoot;
-			     t2 = t2->stack_next)
+			if (__raise_lower_recursion(t, do_lower) == True)
 			{
-				if (FW_W(t2) == FW_W_TRANSIENTFOR(t))
-				{
-					if (IS_ICONIFIED(t2))
-					{
-						break;
-					}
-					if ((!do_lower &&
-					     DO_RAISE_TRANSIENT(t2)) ||
-					    (do_lower &&
-					     DO_LOWER_TRANSIENT(t2))  )
-					{
-						RaiseOrLowerWindow(
-							t2, do_lower, False,
-							False);
-						{
-							return;
-						}
-					}
-				}
+				return;
 			}
 		}
 	}
 
 	if (is_new_window)
 	{
-		no_movement = False;
 		do_move_transients = False;
 	}
 	else
 	{
-		do_move_transients =
-			must_move_transients(t, do_lower, &found_transient);
-
-		/*
-		 * This part implements (Dont)FlipTransient style.
-		 *
-		 * must_move_transients() believes that main should always be
-		 * below its transients. Therefore if it finds a transient but
-		 * does not wish move it, this means main and its superior
-		 * transients are already in the correct position at the top or
-		 * bottom of the layer - depending on do_lower.
-		 */
-		no_movement = found_transient && !do_move_transients;
+		do_move_transients = must_move_transients(t, do_lower);
 	}
-
-	if (!no_movement)
+	if (__restack_window(
+		    t, do_lower, do_move_transients, is_new_window) == True)
 	{
-		/* detach t, so it doesn't make trouble in the loops */
-		remove_window_from_stack_ring(t);
-
-		count = 1;
-		count += get_visible_icon_window_count(t);
-
-		if (do_move_transients)
-		{
-			/* collect the transients in a temp list */
-			tmp_r.stack_prev = &tmp_r;
-			tmp_r.stack_next = &tmp_r;
-			for (t2 = Scr.FvwmRoot.stack_next; t2 != &Scr.FvwmRoot;
-			     t2 = next)
-			{
-				next = t2->stack_next;
-				if ((IS_TRANSIENT(t2)) &&
-				    (FW_W_TRANSIENTFOR(t2) == FW_W(t)) &&
-				    (t2->layer == t->layer) &&
-				    !IS_ICONIFIED(t2))
-				{
-					/* t2 is a transient to lower */
-					count++;
-					count += get_visible_icon_window_count(
-						t2);
-
-					/* unplug it */
-					remove_window_from_stack_ring(t2);
-					/* put it above tmp_r */
-					add_window_to_stack_ring_after(
-						t2, tmp_r.stack_prev);
-				}
-			}
-			if (tmp_r.stack_next == &tmp_r)
-			{
-				do_move_transients = False;
-			}
-		}
-
-		test_layer = t->layer;
-		if (do_lower)
-		{
-			test_layer--;
-		}
-		/* now find the place to reinsert t and friends */
-		for (s = Scr.FvwmRoot.stack_next; s != &Scr.FvwmRoot;
-		     s = s->stack_next)
-		{
-			if (test_layer >= s->layer)
-			{
-				break;
-			}
-		}
-		r = s->stack_prev;
-
-		if (do_move_transients && tmp_r.stack_next != &tmp_r)
-		{
-			/* insert all transients between r and s. */
-			add_windowlist_to_stack_ring_after(&tmp_r, r);
-		}
-
-		/*
-		** Re-insert t - below transients
-		*/
-		add_window_to_stack_ring_after(t, s->stack_prev);
-
-		if (is_new_window && IS_TRANSIENT(t) &&
-		    DO_STACK_TRANSIENT_PARENT(t) && !IS_ICONIFIED(t))
-		{
-			/* now that the new transient is properly positioned in
-			 * the stack ring, raise/lower it again so that its
-			 * parent is raised/lowered too */
-			RaiseOrLowerWindow(t, do_lower, True, False);
-			/* make sure the stacking order is correct - may be the
-			 * sledge-hammer method, but the recursion ist too hard
-			 * to understand. */
-			ResyncXStackingOrder();
-			return;
-		}
-		else
-		{
-			/* restack the windows between r and s */
-			restack_windows(
-				r, s, count, (do_move_transients &&
-					      tmp_r.stack_next != &tmp_r));
-		}
+		return;
 	}
 
 	if (!do_lower)
@@ -712,7 +796,6 @@ static void RaiseOrLowerWindow(
 			raise_over_unmanaged(t);
 		}
 
-
 		/*
 		 * The following is a hack to raise X windows over native
 		 * windows which is needed for some (all ?) X servers running
@@ -747,19 +830,31 @@ static void RaiseOrLowerWindow(
 				XRaiseWindow (dpy, FW_W_FRAME(t2));
 			}
 		}
-
-
 		/*  This needs to be done after all the raise hacks.  */
 		raisePanFrames();
-	}
-	/* If the window has been raised, make sure the decorations are updated
-	 * immediately in case we are in a complex function (e.g. raise,
-	 * unshade). */
-	if (!do_lower)
-	{
+		/* If the window has been raised, make sure the decorations are
+		 * updated immediately in case we are in a complex function
+		 * (e.g. raise, unshade). */
 		XFlush(dpy);
 		handle_all_expose();
 	}
+
+	return;
+}
+
+static void RaiseOrLowerWindow(
+  FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window)
+{
+	FvwmWindow *fw;
+
+	/* clean the auxiliary registers used in stacking transients */
+	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
+	{
+		fw->scratch.i = 0;
+	}
+	__RaiseOrLowerWindow(t, do_lower, allow_recursion, is_new_window);
+
+	return;
 }
 
 /* Raise t and its transients to the top of its layer. For the pager to work
@@ -1004,7 +1099,6 @@ static void BroadcastRestack(FvwmWindow *s1, FvwmWindow *s2)
 	FvwmWindow *t, *t2;
 	int num, i;
 	unsigned long *body, *bp, length;
-	extern Time lastTimestamp;
 
 	if (s2 == &Scr.FvwmRoot)
 	{
@@ -1053,7 +1147,7 @@ static void BroadcastRestack(FvwmWindow *s1, FvwmWindow *s2)
 	*(bp++) = START_FLAG;
 	*(bp++) = M_RESTACK;
 	*(bp++) = length;
-	*(bp++) = lastTimestamp;
+	*(bp++) = fev_get_evtime();
 	for (t2 = t; num != 0; num--, t2 = t2->stack_next)
 	{
 		*(bp++) = FW_W(t2);
@@ -1181,11 +1275,11 @@ void mark_transient_subtree(
 		SET_IN_TRANSIENT_SUBTREE(s, 0);
 		if (IS_TRANSIENT(s) && (layer < 0 || layer == s->layer))
 		{
-			s->pscratch = get_transientfor_fvwmwindow(s);
+			s->scratch.p = get_transientfor_fvwmwindow(s);
 		}
 		else
 		{
-			s->pscratch = NULL;
+			s->scratch.p = NULL;
 		}
 	}
 	/* now loop over the windows and mark the ones we need to move */
@@ -1205,7 +1299,8 @@ void mark_transient_subtree(
 			{
 				continue;
 			}
-			if (DO_ICONIFY_WINDOW_GROUPS(s) && s->wmhints &&
+			if (use_window_group_hint &&
+			    DO_ICONIFY_WINDOW_GROUPS(s) && s->wmhints &&
 			    (s->wmhints->flags & WindowGroupHint) &&
 			    (s->wmhints->window_group != None) &&
 			    (s->wmhints->window_group != FW_W(s)) &&
@@ -1217,11 +1312,11 @@ void mark_transient_subtree(
 			{
 				continue;
 			}
-			r = (FvwmWindow *)s->pscratch;
-			if (do_ignore_icons && r && IS_ICONIFIED(r))
+			if (do_ignore_icons && IS_ICONIFIED(s))
 			{
 				continue;
 			}
+			r = (FvwmWindow *)s->scratch.p;
 			if (IS_TRANSIENT(s))
 			{
 				if (r && IS_IN_TRANSIENT_SUBTREE(r) &&
@@ -1233,6 +1328,8 @@ void mark_transient_subtree(
 				{
 					/* have to move this one too */
 					SET_IN_TRANSIENT_SUBTREE(s, 1);
+					/* used for stacking transients */
+					s->scratch.i += r->scratch.i + 1;
 					/* need another scan through the list */
 					is_finished = False;
 					continue;
@@ -1290,6 +1387,7 @@ void new_layer(FvwmWindow *fw, int layer)
 	int add_after_layer;
 	int count;
 	int old_layer;
+	Bool do_lower;
 
 	fw = get_transientfor_top_fvwmwindow(fw);
 	if (layer == fw->layer)
@@ -1312,11 +1410,13 @@ void new_layer(FvwmWindow *fw, int layer)
 	{
 		/* lower below the windows in the new (lower) layer */
 		add_after_layer = layer;
+		do_lower = True;
 	}
 	else
 	{
 		/* raise above the windows in the new (higher) layer */
 		add_after_layer = layer + 1;
+		do_lower = False;
 	}
 	/* find the place to insert the windows */
 	for (target = Scr.FvwmRoot.stack_next; target != &Scr.FvwmRoot;
@@ -1344,7 +1444,8 @@ void new_layer(FvwmWindow *fw, int layer)
 	}
 	/* move the windows without modifying their stacking order */
 	restack_windows(
-		list_head.stack_next->stack_prev, target, count, (count > 1));
+		list_head.stack_next->stack_prev, target, count, (count > 1),
+		do_lower);
 	focus_grab_buttons_on_layer(layer);
 	focus_grab_buttons_on_layer(old_layer);
 
@@ -1357,9 +1458,9 @@ void new_layer(FvwmWindow *fw, int layer)
 /*  RBW - 11/13/1998 - 2 new fields to init - stacking order chain.  */
 void init_stack_and_layers(void)
 {
-	Scr.BottomLayer  = DEFAULT_BOTTOM_LAYER;
+	Scr.BottomLayer	 = DEFAULT_BOTTOM_LAYER;
 	Scr.DefaultLayer = DEFAULT_DEFAULT_LAYER;
-	Scr.TopLayer     = DEFAULT_TOP_LAYER;
+	Scr.TopLayer	 = DEFAULT_TOP_LAYER;
 	Scr.FvwmRoot.stack_next = &Scr.FvwmRoot;
 	Scr.FvwmRoot.stack_prev = &Scr.FvwmRoot;
 	set_layer(&Scr.FvwmRoot, DEFAULT_ROOT_WINDOW_LAYER);
@@ -1369,11 +1470,11 @@ void init_stack_and_layers(void)
 
 Bool is_above_unmanaged(FvwmWindow *fw, Window *umtop)
 {
-/*
-          Chase through the entire stack of the server's windows looking
-        for any unmanaged window that's higher than the target.
-          Called from raise_over_unmanaged and is_on_top_of_layer.
-*/
+	/*
+	  Chase through the entire stack of the server's windows looking
+	  for any unmanaged window that's higher than the target.
+	  Called from raise_over_unmanaged and is_on_top_of_layer.
+	*/
 	Bool ontop = True;
 
 
@@ -1406,23 +1507,23 @@ Bool is_above_unmanaged(FvwmWindow *fw, Window *umtop)
 		{
 			continue;
 		}
-                /*
-                    Don't forget to ignore the hidden frame resizing windows...
-                */
+		/*
+		  Don't forget to ignore the hidden frame resizing windows...
+		*/
 		if (wa.override_redirect == True
-                    && wa.class != InputOnly
+		    && wa.class != InputOnly
 		    && tops[i] != Scr.NoFocusWin
-                    && (! is_frame_hide_window(tops[i]))
-                    )
+		    && (! is_frame_hide_window(tops[i]))
+			)
 		{
 			OR_Above = tops[i];
 		}
 	} /* end for */
 
 	if (OR_Above)  {
-        *umtop = OR_Above;
-        ontop = False;
-        }
+		*umtop = OR_Above;
+		ontop = False;
+	}
 
 	XFree (tops);
 
@@ -1441,6 +1542,10 @@ static Bool is_on_top_of_layer_ignore_rom(FvwmWindow *fw)
 		/* stack ring members are no longer valid */
 		return False;
 	}
+	if (DO_RAISE_TRANSIENT(fw))
+	{
+		mark_transient_subtree(fw, fw->layer, MARK_RAISE, True, False);
+	}
 	for (t = fw->stack_prev; t != &Scr.FvwmRoot; t = t->stack_prev)
 	{
 		if (t->layer > fw->layer)
@@ -1457,8 +1562,8 @@ static Bool is_on_top_of_layer_ignore_rom(FvwmWindow *fw)
 		 * would be considered on top. */
 		if (Scr.bo.RaiseOverUnmanaged || overlap(fw, t))
 		{
-			if (!IS_TRANSIENT(t) || !DO_RAISE_TRANSIENT(fw) ||
-			    FW_W_TRANSIENTFOR(t) != FW_W(fw))
+			if (!DO_RAISE_TRANSIENT(fw) ||
+			    (!IS_IN_TRANSIENT_SUBTREE(t) && t != fw))
 			{
 				ontop = False;
 				break;
@@ -1471,95 +1576,91 @@ static Bool is_on_top_of_layer_ignore_rom(FvwmWindow *fw)
 
 Bool _is_on_top_of_layer(FvwmWindow *fw, Bool client_entered)
 {
-Window  junk;
-Bool  ontop     = False;
+	Window	junk;
+	Bool  ontop	= False;
 	if (Scr.bo.RaiseOverUnmanaged)
 	{
 
 #define EXPERIMENTAL_ROU_HANDLING
 #ifdef EXPERIMENTAL_ROU_HANDLING
-/*
-    RBW - 2002/08/15 - 
-          RaiseOverUnmanaged adds some overhead. The only way to let our
-        caller know for sure whether we need to grab the mouse buttons
-        because we may need to raise this window is to query the
-        server's tree and look for any override_redirect windows above
-        this one.
-          But this function is called far too often to do this every
-        time. Only if the window is at the top of the FvwmWindow stack do
-        we need more information from the server; and then only at the
-        last moment in HandleEnterNotify when we really need to know
-        whether a raise will be needed if the user clicks in the client
-        window. is_on_top_of_layer_and_above_unmanaged is called in that
-        case.
-*/
-	if (is_on_top_of_layer_ignore_rom(fw))
+		/*
+		  RBW - 2002/08/15 -
+		  RaiseOverUnmanaged adds some overhead. The only way to let our
+		  caller know for sure whether we need to grab the mouse buttons
+		  because we may need to raise this window is to query the
+		  server's tree and look for any override_redirect windows above
+		  this one.
+		  But this function is called far too often to do this every
+		  time. Only if the window is at the top of the FvwmWindow
+		  stack do we need more information from the server; and then
+		  only at the last moment in HandleEnterNotify when we really
+		  need to know whether a raise will be needed if the user
+		  clicks in the client window.
+		  is_on_top_of_layer_and_above_unmanaged is called in that case.
+		*/
+		if (is_on_top_of_layer_ignore_rom(fw))
 		{
-                if (client_entered)  
-                /* FIXME! - perhaps we should only do if MFCR  */
-                        {
+			if (client_entered)
+				/* FIXME! - perhaps we should only do if MFCR */
+			{
 #ifdef ROUDEBUG
-                        printf("RBW-iotol  - %8.8lx is on top, checking server tree.  ***\n", FW_W_CLIENT(fw));
+				printf("RBW-iotol  - %8.8lx is on top,"
+				       " checking server tree.  ***\n",
+				       FW_W_CLIENT(fw));
 #endif
-		        ontop = is_above_unmanaged(fw, &junk);
+				ontop = is_above_unmanaged(fw, &junk);
 #ifdef ROUDEBUG
-                        printf("         returning %d\n", (int) ontop);
+				printf("	 returning %d\n", (int) ontop);
 #endif
-                        }
-                        else  
-                        {
+			}
+			else
+			{
 #ifdef ROUDEBUG
-                        printf("RBW-iotol  - %8.8lx is on top, *** NOT checking server tree.\n", FW_W_CLIENT(fw));
+				printf("RBW-iotol  - %8.8lx is on top,"
+				       " *** NOT checking server tree.\n",
+				       FW_W_CLIENT(fw));
 #endif
-                        ontop = True;
-                        }
-		return ontop;
+				ontop = True;
+			}
+			return ontop;
 		}
 		else
 		{
-		return False;
+			return False;
 		}
 #else
-		return False;   /*  Old pre-2002/08/22 handling.  */
+		return False;	/*  Old pre-2002/08/22 handling.  */
 #endif
 	}
-        else
-        {
-	return is_on_top_of_layer_ignore_rom(fw);
-        }
+	else
+	{
+		return is_on_top_of_layer_ignore_rom(fw);
+	}
 }
 
 
 Bool is_on_top_of_layer(FvwmWindow *fw)
 {
-        return _is_on_top_of_layer(fw, False);
+	return _is_on_top_of_layer(fw, False);
 }
 
 Bool is_on_top_of_layer_and_above_unmanaged(FvwmWindow *fw)
 {
-        return _is_on_top_of_layer(fw, True);
+	return _is_on_top_of_layer(fw, True);
 }
 
 /* ----------------------------- built in functions ------------------------ */
 
 void CMD_Raise(F_CMD_ARGS)
 {
-	if (DeferExecution(eventp,&w,&fw,&context, CRS_SELECT,ButtonRelease))
-	{
-		return;
-	}
-	RaiseWindow(fw);
+	RaiseWindow(exc->w.fw);
 
 	return;
 }
 
 void CMD_Lower(F_CMD_ARGS)
 {
-	if (DeferExecution(eventp,&w,&fw,&context, CRS_SELECT, ButtonRelease))
-	{
-		return;
-	}
-	LowerWindow(fw);
+	LowerWindow(exc->w.fw);
 
 	return;
 }
@@ -1567,14 +1668,8 @@ void CMD_Lower(F_CMD_ARGS)
 void CMD_RaiseLower(F_CMD_ARGS)
 {
 	Bool ontop;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(eventp,&w,&fw,&context, CRS_SELECT,ButtonRelease))
-	{
-#ifdef DEBUG_STACK_RING
-		dump_stack_ring();
-#endif
-		return;
-	}
 	ontop = is_on_top_of_layer_ignore_rom(fw);
 	if (ontop)
 	{
@@ -1592,17 +1687,12 @@ void CMD_Layer(F_CMD_ARGS)
 {
 	int n, layer, val[2];
 	char *token;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
-	{
-		return;
-	}
 	if (fw == NULL)
 	{
 		return;
 	}
-
 	token = PeekToken(action, NULL);
 	if (StrEquals("default", token))
 	{
