@@ -238,17 +238,10 @@ void send_clientmessage (Display *disp, Window w, Atom a, Time timestamp);
 /* not really a wrapper, but useful and X related */
 void PrintXErrorAndCoredump(Display *dpy, XErrorEvent *error, char *MyName);
 
-/*
- * This function determines the location of the mouse pointer from the event
- * if possible, if not it queries the X server. Returns False if it had to
- * query the server and the call failed.
- */
-Bool GetLocationFromEventOrQuery(Display *dpy, Window w, XEvent *eventp,
-				 int *ret_x, int *ret_y);
 /*
  * Return the subwindow member of an event if the event type has one.
  */
-Window GetSubwindowFromEvent(Display *dpy, XEvent *eventp);
+Window GetSubwindowFromEvent(Display *dpy, const XEvent *eventp);
 
 /***********************************************************************
  * Wrappers around Xrm routines (XResources.c)
@@ -401,6 +394,7 @@ int GetEqualSizeChildren(
 #define C_SIDEBAR     (C_SB_LEFT|C_SB_RIGHT|C_SB_TOP|C_SB_BOTTOM)
 #define C_RALL        (C_R1|C_R2|C_R3|C_R4|C_R5)
 #define C_LALL        (C_L1|C_L2|C_L3|C_L4|C_L5)
+#define C_DECOR       (C_LALL|C_RALL|C_TITLE|C_FRAME|C_SIDEBAR)
 #define C_ALL         (C_WINDOW|C_TITLE|C_ICON|C_ROOT|C_FRAME|C_SIDEBAR|\
 		       C_LALL|C_RALL|C_EWMH_DESKTOP)
 
@@ -419,7 +413,7 @@ typedef struct Binding
 {
   BindingType type;       /* Is it a mouse, key, or stroke binding */
   STROKE_CODE(void *Stroke_Seq;) /* stroke sequence */
-  int Button_Key;         /* Mouse Button number of Keycode */
+  int Button_Key;         /* Mouse Button number or Keycode */
   char *key_name;         /* In case of keycode, give the key_name too */
   int Context;            /* Contex is Fvwm context, ie titlebar, frame, etc */
   int Modifier;           /* Modifiers for keyboard state */
