@@ -27,8 +27,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#define DRAW_RELIEF      0
+#define DRAW_CLEAN       1
+#define DRAW_ALL         2
+#define DRAW_FORCE       3
+#define DRAW_DESK_RELIEF 4
+
 void RelieveButton(Window,int,int,int,int,int,Pixel,Pixel,int);
 void MakeButton(button_info*);
-void RedrawButton(button_info*,int);
-void DrawTitle(button_info*,Window,GC);
+void RedrawButton(button_info*, int draw, XEvent *pev);
+void DrawTitle(button_info*, Window, GC gc ,XEvent *pev);
 
