@@ -36,7 +36,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/Module.h"
-#include "libs/Picture.h"
+#include "libs/PictureBase.h"
 
 
 extern int fvwm_fd[2];
@@ -109,7 +109,7 @@ convert_label_with_ampersand (const gchar * _label,
 	    break;
 	if (*p == '&') {
 	    memmove (p, p + 1, strlen (p));
-	    if (*p == '&')	/* use && for an actual & */
+	    if (*p == '&')      /* use && for an actual & */
 		continue;
 	    *underbar_pos = (unsigned long) p - (unsigned long) label;
 	    *accelerator_key = *p;
@@ -167,7 +167,7 @@ menu_item_new_with_pixmap_and_label (char *file, char *l_label, char *r_label)
   int w, h;
 #endif
 
-  path = findImageFile (file ? file : "", image_path, R_OK);
+  path = PictureFindImageFile (file ? file : "", image_path, R_OK);
 
   if (path == NULL)
     {
