@@ -1327,9 +1327,12 @@ void read_in_resources (char *file)
 	  continue;
 	}
 	for ( i = 0; i < NUM_CONTEXTS; i++ ) {
-	  SET_MANAGER(manager, colorsets[i], n);
-	  AllocColorset(n);
+		if (i != DEFAULT && i != TITLE_CONTEXT)
+			SET_MANAGER(manager, colorsets[i], -2);
+		else
+			SET_MANAGER(manager, colorsets[i], n);
 	}
+	AllocColorset(n);
       }
       else if (!strcasecmp (option1, "background")) {
 	p = read_next_cmd (READ_ARG);
@@ -1435,6 +1438,27 @@ void read_in_resources (char *file)
 	ConsoleDebug (CONFIG, "Setting followfocus to: %d\n", i);
 	SET_MANAGER (manager, followFocus, i);
       }
+      else if (!strcasecmp (option1, "showtransient")) {
+       p = read_next_cmd (READ_ARG);
+       if (!p) {
+	 ConsoleMessage ("Bad line: %s\n", current_line);
+	 ConsoleMessage ("Need argument to showtransient\n");
+	 continue;
+       }
+       if (!strcasecmp (p, "true")) {
+	 i = 1;
+       }
+       else if (!strcasecmp (p, "false")) {
+	 i = 0;
+       }
+       else {
+	 ConsoleMessage ("Bad line: %s\n", current_line);
+	 ConsoleMessage ("What is this: %s?\n", p);
+	 continue;
+       }
+       ConsoleDebug (CONFIG, "Setting showtransient to: %d\n", i);
+       SET_MANAGER (manager, showtransient, i);
+      }
       else if (!strcasecmp (option1, "showonlyiconic")) {
        p = read_next_cmd (READ_ARG);
        if (!p) {
