@@ -21,17 +21,17 @@
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/PictureUtils.h"
+#include "libs/FRender.h"
+#include "libs/FRenderInit.h"
 
-static char const rcsid[] =
-  "$Id: x.c,v 1.57 2002/01/08 14:21:59 olicha Exp $";
-
-#define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask)
+#define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|\
+	EnterWindowMask|LeaveWindowMask)
 
 Display *theDisplay;
 Window theRoot;
 int theScreen;
 static Atom _XA_WM_DEL_WIN;
-extern char *MyName;
 
 static enum {
   NOT_GRABBED = 0,
@@ -56,8 +56,10 @@ static void grab_pointer (WinManager *man)
   return;
 }
 
+
 static int lookup_color (char *name, Pixel *ans)
 {
+#if 0
   XColor color;
 
   color.pixel = 0;
@@ -71,9 +73,13 @@ static int lookup_color (char *name, Pixel *ans)
   }
   *ans = color.pixel;
   return 1;
+#else
+  /* use the lib */
+  *ans = GetSimpleColor(name);
+  return 1;
+#endif
 }
 
-
 WinManager *find_windows_manager (Window win)
 {
   int i;
@@ -194,13 +200,14 @@ void xevent_loop (void)
   static int flag = 0;
   WinManager *man;
   Bool force_redraw = False;
+  static int ex=10000, ey=10000, ex2=0, ey2=0;
 
   if (flag == 0) {
     flag = 1;
     ConsoleDebug (X11, "A virgin event loop\n");
   }
-  while (XPending (theDisplay)) {
-    XNextEvent (theDisplay, &theEvent);
+  while (FPending(theDisplay)) {
+    FNextEvent(theDisplay, &theEvent);
     if (theEvent.type == MappingNotify) {
       ConsoleDebug (X11, "XEVENT: MappingNotify\n");
       continue;
@@ -252,12 +259,51 @@ void xevent_loop (void)
 
     case Expose:
       ConsoleDebug (X11, "XEVENT: Expose\n");
-      if (theEvent.xexpose.count == 0) {
-	man_exposed (man, &theEvent);
-	draw_manager (man);
-	if (globals.transient) {
-	  grab_pointer (man);
-	}
+#if 0
+      fprintf (stderr, "Expose: %i,%i,%i,%i %i\n",
+	       theEvent.xexpose.x, theEvent.xexpose.y, theEvent.xexpose.width,
+	       theEvent.xexpose.height, theEvent.xexpose.count);
+#endif
+      ex = min(ex, theEvent.xexpose.x);
+      ey = min(ey, theEvent.xexpose.y);
+      ex2 = max(ex2, theEvent.xexpose.x + theEvent.xexpose.width);
+      ey2=max(ey2 , theEvent.xexpose.y + theEvent.xexpose.height);
+
+      /* we use "clip redrawing": but this speedup redrawing without
+       * adding to much flickering */
+      while (FCheckTypedWindowEvent(
+	      theDisplay, theEvent.xany.window, Expose, &theEvent))
+      {
+#if 0
+	      fprintf (stderr, "\t\tExpose Purge: %i,%i,%i,%i %i\n",
+		       theEvent.xexpose.x, theEvent.xexpose.y,
+		       theEvent.xexpose.width, theEvent.xexpose.height,
+		       theEvent.xexpose.count);
+#endif
+	      ex = min(ex, theEvent.xexpose.x);
+	      ey = min(ey, theEvent.xexpose.y);
+	      ex2 = max(ex2, theEvent.xexpose.x + theEvent.xexpose.width);
+	      ey2= max(ey2 , theEvent.xexpose.y + theEvent.xexpose.height);
+      }
+#if 0
+      fprintf (
+	      stderr, "\tExpose Done: %i,%i,%i,%i %i\n",
+	      ex, ey, ex2-ex, ey2-ey, theEvent.xexpose.count);
+#endif
+      if (theEvent.xexpose.count == 0)
+      {
+	      theEvent.xexpose.x = ex;
+	      theEvent.xexpose.y = ey;
+	      theEvent.xexpose.width = ex2 - ex;
+	      theEvent.xexpose.height = ey2 -ey;
+	      man_exposed (man, &theEvent);
+	      draw_manager (man);
+	      if (globals.transient) {
+		      grab_pointer (man);
+	      }
+	      
+	      ex = ey = 10000;
+	      ex2 = ey2 = 0;
       }
       break;
 
@@ -295,6 +341,7 @@ void xevent_loop (void)
        * windows must be refreshed since the server won't do this,
        * we must not miss ones of these */
       Bool moved = theEvent.xconfigure.send_event;
+      Bool recreate_bg = False;
 
       ConsoleDebug (X11, "\tcurrent geometry: %d %d %d %d\n",
 		    man->geometry.x, man->geometry.y,
@@ -305,8 +352,8 @@ void xevent_loop (void)
 
       saveEvent = theEvent;
       /* eat up all but last ConfigureNotify events */
-      while (XPending(theDisplay) &&
-	     XCheckTypedEvent(theDisplay, ConfigureNotify, &theEvent))
+      while (FPending(theDisplay) &&
+	     FCheckTypedEvent(theDisplay, ConfigureNotify, &theEvent))
       {
 	saveEvent = theEvent;
 	/* check for movement on all events */
@@ -314,61 +361,95 @@ void xevent_loop (void)
 	  moved = True;
       }
       theEvent = saveEvent;
-	      
-      if (man->geometry.dir & GROW_FIXED)
+
+#if 0
+      fprintf (stderr, "Configure: %i,%i,%i,%i %i\n",
+	       theEvent.xconfigure.x, theEvent.xconfigure.y,
+	       theEvent.xconfigure.width, theEvent.xconfigure.height,
+	       moved);
+#endif
+       if ((man->geometry.width != theEvent.xconfigure.width ||
+	   man->geometry.height != theEvent.xconfigure.height ||
+	   (moved && CSET_IS_TRANSPARENT(man->colorsets[DEFAULT]))) &&
+	  !CSET_IS_TRANSPARENT_PR_PURE(man->colorsets[DEFAULT]))
       {
-	man->geometry.rows =
-	  theEvent.xconfigure.height / man->geometry.boxheight;
-	if (man->geometry.rows < 1)
-	  man->geometry.rows = 1;
-	man->geometry.cols =
-	  (man->buttons.num_windows - 1) / man->geometry.rows + 1;
-	force_redraw = 1;
+	      recreate_bg = True;
+	      if (moved)
+	      {
+		      man->geometry.x = theEvent.xconfigure.x;
+		      man->geometry.y = theEvent.xconfigure.y;
+	      }
       }
-      if ( man->geometry.width != theEvent.xconfigure.width ||
-           man->geometry.height != theEvent.xconfigure.height) {
-        man->geometry.width = theEvent.xconfigure.width;
-        man->geometry.height = theEvent.xconfigure.height;
-      ConsoleDebug (X11, "\tcurrent geometry: %d %d %d %d\n",
-		    man->geometry.x, man->geometry.y,
-		    man->geometry.width, man->geometry.height);
-        if (man->colorsets[DEFAULT] >= 0) {
-	  if (man->pixmap[DEFAULT] && man->pixmap[DEFAULT] != ParentRelative)
-	    XFreePixmap(theDisplay, man->pixmap[DEFAULT]);
-	  if (Colorset[man->colorsets[DEFAULT]].pixmap) {
-	    man->pixmap[DEFAULT] =
-	      CreateBackgroundPixmap(
-		theDisplay, man->theWindow, man->geometry.width,
-		man->geometry.height,
-		&Colorset[man->colorsets[DEFAULT]],
-		Pdepth, man->backContext[DEFAULT], False);
-	    XSetTile(theDisplay, man->backContext[DEFAULT],
-		     man->pixmap[DEFAULT]);
-	    XSetFillStyle(theDisplay, man->backContext[DEFAULT], FillTiled);
-	  } else {
-	    man->pixmap[DEFAULT] = None;
-	    XSetFillStyle(theDisplay, man->backContext[DEFAULT], FillSolid);
-	  }
-        }
-	force_redraw = 1;
-      }
-      /* must refresh transparent windows when moved */
-      if (moved && man->pixmap[DEFAULT] == ParentRelative)
+
+      if (man->geometry.width != theEvent.xconfigure.width ||
+	  man->geometry.height != theEvent.xconfigure.height)
       {
-	man->geometry.x = theEvent.xconfigure.x;
-	man->geometry.y = theEvent.xconfigure.y;
-        XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
+	      if (man->geometry.dir & GROW_FIXED)
+	      {
+		      man->geometry.rows =
+			      theEvent.xconfigure.height / 
+			      man->geometry.boxheight;
+		      if (man->geometry.rows < 1)
+		      {
+			      man->geometry.rows = 1;
+		      }
+		      man->geometry.cols =
+			      (man->buttons.num_windows - 1) /
+			      man->geometry.rows+1;
+	      }
+	      man->geometry.width = theEvent.xconfigure.width;
+	      man->geometry.height = theEvent.xconfigure.height;
+	      ConsoleDebug (X11, "\tcurrent geometry: %d %d %d %d\n",
+			    man->geometry.x, man->geometry.y,
+			    man->geometry.width, man->geometry.height);
+	      force_redraw = 1;
+      }
+
+      if (recreate_bg)
+      {
+	      if (man->colorsets[DEFAULT] >= 0)
+	      {
+		      recreate_background(man, DEFAULT);
+	      }
+	      else
+	      {
+		      recreate_bg = False; 
+	      }
+      }
+
+      /* must refresh transparent buttons when:
+       * - moved and buttons is root transparent
+       * - bg change and  butons is parental relative */
+      if (moved || recreate_bg)
+      {
+	      Bool r = False;
+
+	      man->geometry.x = theEvent.xconfigure.x;
+	      man->geometry.y = theEvent.xconfigure.y;
+	      recreate_transparent_bgs(man);
+	      if (!force_redraw)
+	      {
+		      r = draw_transparent_buttons(
+			      man, (moved && !recreate_bg), False);
+		      if (r)
+		      {
+			      XSync(theDisplay, False);
+		      }
+		      break;
+	      }
       }
 
       set_manager_width (man, theEvent.xconfigure.width);
       ConsoleDebug (X11, "\tboxwidth = %d\n", man->geometry.boxwidth);
       if (force_redraw)
       {
-	force_manager_redraw(man);
-	force_redraw = 0;
+	      force_manager_redraw(man);
+	      force_redraw = 0;
       }
       else
-	draw_manager (man);
+      {
+	      /* draw_manager (man);*/
+      }
 
       break;
     }
@@ -444,8 +525,8 @@ static void set_window_properties (Window win, char *name, char *icon,
     return;
   }
 
-  class.res_name = Module + 1;
-  class.res_class = "FvwmModule";
+  class.res_name = MyName;
+  class.res_class = "FvwmIconMan";
 
 
   XSetWMProperties (theDisplay, win, &win_name, &win_icon, NULL, 0,
@@ -459,7 +540,7 @@ static int load_default_context_fore (WinManager *man, int i)
 {
   int j = 0;
 
-  if (Pdepth > 2)
+  if (!PictureUseBWOnly())
     j = 1;
 
   ConsoleDebug (X11, "Loading: %s\n", contextDefaults[i].backcolor[j]);
@@ -471,7 +552,7 @@ static int load_default_context_back (WinManager *man, int i)
 {
   int j = 0;
 
-  if (Pdepth > 2)
+  if (!PictureUseBWOnly())
     j = 1;
 
   ConsoleDebug (X11, "Loading: %s\n", contextDefaults[i].backcolor[j]);
@@ -506,12 +587,7 @@ void X_init_manager (int man_id)
   WinManager *man;
   unsigned int width, height;
   int i, x, y, geometry_mask;
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#endif
+
   ConsoleDebug (X11, "In X_init_manager\n");
 
   man = &globals.managers[man_id];
@@ -526,41 +602,14 @@ void X_init_manager (int man_id)
 
   ConsoleDebug (X11, "boxwidth = %d\n", man->geometry.boxwidth);
 
-  if (man->fontname) {
-#ifdef I18N_MB
-    man->ButtonFontset = XCreateFontSet (theDisplay, man->fontname, &ml, &mc, &ds);
-    if (!man->ButtonFontset) {
-      if (!(man->ButtonFontset = XCreateFontSet (theDisplay, FONT_STRING, &ml, &mc, &ds))) {
-        ConsoleMessage ("Can't get fontset\n");
-        ShutMeDown (1);
-      }
-    }
-    XFontsOfFontSet(man->ButtonFontset,&fs_list,&ml);
-    man->ButtonFont = fs_list[0];
-#else
-    man->ButtonFont = XLoadQueryFont (theDisplay, man->fontname);
-    if (!man->ButtonFont) {
-      if (!(man->ButtonFont = XLoadQueryFont (theDisplay, FONT_STRING))) {
-	ConsoleMessage ("Can't get font\n");
-	ShutMeDown (1);
-      }
-    }
-#endif
-  }
-  else {
-#ifdef I18N_MB
-    if (!(man->ButtonFontset = XCreateFontSet (theDisplay, FONT_STRING, &ml, &mc, &ds))) {
-      ConsoleMessage ("Can't get fontset\n");
-      ShutMeDown (1);
-    }
-    XFontsOfFontSet(man->ButtonFontset,&fs_list,&ml);
-    man->ButtonFont = fs_list[0];
-#else
-    if (!(man->ButtonFont = XLoadQueryFont (theDisplay, FONT_STRING))) {
-      ConsoleMessage ("Can't get font\n");
-      ShutMeDown (1);
-    }
-#endif
+  if (man->fontname == NULL)
+    man->FButtonFont = FlocaleLoadFont(theDisplay, FONT_STRING, MyName);
+  else
+    man->FButtonFont = FlocaleLoadFont(theDisplay, man->fontname, MyName)  ;
+  if (man->FButtonFont == NULL)
+  {
+    ConsoleMessage ("Can't get font, exiting\n");
+    ShutMeDown (1);
   }
 
   for ( i = 0; i < NUM_CONTEXTS; i++ ) {
@@ -568,12 +617,15 @@ void X_init_manager (int man_id)
     if (man->colorsets[i] >= 0) {
       man->backcolor[i] = Colorset[man->colorsets[i]].bg;
     }
+    else if (man->colorsets[i] == -2) {
+      man->backcolor[i] = Colorset[man->colorsets[DEFAULT]].bg;
+    }
     else if (man->backColorName[i]) {
       if (!lookup_color (man->backColorName[i], &man->backcolor[i])) {
-        if (!load_default_context_back (man, i)) {
+	if (!load_default_context_back (man, i)) {
 	  ConsoleMessage ("Can't load %s background color\n",
 			  contextDefaults[i].name);
-        }
+	}
       }
     }
     else if (!load_default_context_back (man, i)) {
@@ -584,12 +636,15 @@ void X_init_manager (int man_id)
     if (man->colorsets[i] >= 0) {
       man->forecolor[i] = Colorset[man->colorsets[i]].fg;
     }
+    else if (man->colorsets[i] == -2) {
+      man->forecolor[i] = Colorset[man->colorsets[DEFAULT]].fg;
+    }
     else if (man->foreColorName[i]) {
       if (!lookup_color (man->foreColorName[i], &man->forecolor[i])) {
-        if (!load_default_context_fore (man, i)) {
-    	ConsoleMessage ("Can't load %s foreground color\n",
+	if (!load_default_context_fore (man, i)) {
+	ConsoleMessage ("Can't load %s foreground color\n",
 			contextDefaults[i].name);
-        }
+	}
       }
     }
     else if (!load_default_context_fore (man, i)) {
@@ -597,10 +652,14 @@ void X_init_manager (int man_id)
 		      contextDefaults[i].name);
     }
 
-    if (Pdepth > 2) {
+    if (!PictureUseBWOnly()) {
       if (man->colorsets[i] >= 0) {
-        man->shadowcolor[i] = Colorset[man->colorsets[i]].shadow;
-        man->hicolor[i] = Colorset[man->colorsets[i]].hilite;
+	man->shadowcolor[i] = Colorset[man->colorsets[i]].shadow;
+	man->hicolor[i] = Colorset[man->colorsets[i]].hilite;
+      }
+      else if (man->colorsets[i] == -2) {
+	man->shadowcolor[i] = Colorset[man->colorsets[DEFAULT]].shadow;
+	man->hicolor[i] = Colorset[man->colorsets[DEFAULT]].hilite;
       }
       else
       {
@@ -610,13 +669,12 @@ void X_init_manager (int man_id)
     }
   }
 
-  man->fontheight = man->ButtonFont->ascent +
-    man->ButtonFont->descent;
+  man->fontheight = man->FButtonFont->height;
 
   /* silly hack to guess the minimum char width of the font
      doesn't have to be perfect. */
 
-  man->fontwidth = XTextWidth (man->ButtonFont, ".", 1);
+  man->fontwidth = FlocaleTextWidth (man->FButtonFont, ".", 1);
 
   /* First:  get button geometry
      Second: get geometry from geometry string
@@ -640,7 +698,7 @@ void X_init_manager (int man_id)
     if (val & HeightValue)
     {
       if (height >= man->geometry.boxheight - 4)
-        man->geometry.boxheight = height;
+	man->geometry.boxheight = height;
     }
   }
   FScreenGetScrRect(NULL, FSCREEN_PRIMARY, &man->managed_g.x, &man->managed_g.y,
@@ -706,7 +764,7 @@ void X_init_manager (int man_id)
     unsigned int ujunk;
     fscreen_scr_arg fscr;
 
-    if (XQueryPointer(
+    if (FQueryPointer(
 	  theDisplay, theRoot, &dummyroot, &dummychild, &man->geometry.x,
 	  &man->geometry.y, &junk, &junk, &ujunk) == False)
     {
@@ -869,54 +927,59 @@ void create_manager_window (int man_id)
   man->off_y = 0;
 
   for (i = 0; i < NUM_CONTEXTS; i++) {
-    man->backContext[i] =
-      fvwmlib_XCreateGC (theDisplay, man->theWindow, gcmask, &gcval);
-    XSetForeground (theDisplay, man->backContext[i], man->backcolor[i]);
-    XSetLineAttributes (theDisplay, man->backContext[i], line_width,
-			line_style, cap_style,
-			join_style);
+	  if (man->colorsets[i] != -2)
+	  {
+		  man->backContext[i] =
+			  fvwmlib_XCreateGC(
+				  theDisplay, man->theWindow, gcmask, &gcval);
+		  XSetForeground (
+			  theDisplay, man->backContext[i], man->backcolor[i]);
+		  XSetLineAttributes (
+			  theDisplay, man->backContext[i], line_width,
+			  line_style, cap_style,
+			  join_style);
+	  }
+	  man->hiContext[i] = fvwmlib_XCreateGC(
+		  theDisplay, man->theWindow, gcmask, &gcval);
+	  if (man->FButtonFont->font != NULL)
+		  XSetFont(
+			  theDisplay, man->hiContext[i],
+			  man->FButtonFont->font->fid);
+	  XSetForeground (theDisplay, man->hiContext[i], man->forecolor[i]);
 
-    man->hiContext[i] =
-      fvwmlib_XCreateGC (theDisplay, man->theWindow, gcmask, &gcval);
-    XSetFont (theDisplay, man->hiContext[i], man->ButtonFont->fid);
-    XSetForeground (theDisplay, man->hiContext[i], man->forecolor[i]);
+	  gcmask = GCForeground | GCBackground;
+	  gcval.foreground = man->backcolor[i];
+	  gcval.background = man->forecolor[i];
+	  man->flatContext[i] = fvwmlib_XCreateGC(
+		  theDisplay, man->theWindow, gcmask, &gcval);
+	  if (!PictureUseBWOnly())
+	  {
+		  gcmask = GCForeground | GCBackground;
+		  gcval.foreground = man->hicolor[i];
+		  gcval.background = man->backcolor[i];
+		  man->reliefContext[i] = fvwmlib_XCreateGC (
+			  theDisplay, man->theWindow, gcmask, &gcval);
 
-    gcmask = GCForeground | GCBackground;
-    gcval.foreground = man->backcolor[i];
-    gcval.background = man->forecolor[i];
-    man->flatContext[i] = fvwmlib_XCreateGC (theDisplay, man->theWindow,
-						 gcmask, &gcval);
-    if (Pdepth > 2) {
-      gcmask = GCForeground | GCBackground;
-      gcval.foreground = man->hicolor[i];
-      gcval.background = man->backcolor[i];
-      man->reliefContext[i] = fvwmlib_XCreateGC (theDisplay, man->theWindow,
-						     gcmask, &gcval);
-
-      gcmask = GCForeground | GCBackground;
-      gcval.foreground = man->shadowcolor[i];
-      gcval.background = man->backcolor[i];
-      man->shadowContext[i] = fvwmlib_XCreateGC (theDisplay, man->theWindow,
-						     gcmask, &gcval);
-    }
-    if (man->pixmap[i] && man->pixmap[i] != ParentRelative)
-      XFreePixmap(theDisplay, man->pixmap[i]);
-    if (man->colorsets[i] >= 0 && Colorset[man->colorsets[i]].pixmap) {
-      man->pixmap[i] = CreateBackgroundPixmap(theDisplay, man->theWindow,
-                       man->geometry.width, man->geometry.height,
-                       &Colorset[man->colorsets[i]],
-                       Pdepth, man->backContext[i], False);
-      XSetTile(theDisplay, man->backContext[i], man->pixmap[i]);
-      XSetFillStyle(theDisplay, man->backContext[i], FillTiled);
-      if (i == DEFAULT)
-      {
-	XSetWindowBackgroundPixmap(theDisplay, man->theWindow,
-				   man->pixmap[i]);
-      }
-    } else {
-      man->pixmap[i] = None;
-      XSetFillStyle(theDisplay, man->backContext[i], FillSolid);
-    }
+		  gcmask = GCForeground | GCBackground;
+		  gcval.foreground = man->shadowcolor[i];
+		  gcval.background = man->backcolor[i];
+		  man->shadowContext[i] = fvwmlib_XCreateGC (
+			  theDisplay, man->theWindow, gcmask, &gcval);
+	  }
+	  if (man->colorsets[i] >= 0)
+	  {
+		  recreate_background(man, i);
+	  }
+	  else if (man->colorsets[i] != -2)
+	  {
+		  man->pixmap[i] = None;
+		  XSetFillStyle(theDisplay, man->backContext[i], FillSolid);
+		  if (i == DEFAULT)
+		  {
+			  XSetWindowBackground (
+				  theDisplay, man->theWindow, man->backcolor[i]);
+		  }
+	  }
   }
 
   set_window_properties (man->theWindow, man->titlename,
@@ -938,6 +1001,9 @@ static int handle_error (Display *d, XErrorEvent *ev)
     break;
   }
 
+  if (FRenderGetErrorCodeBase() + FRenderBadPicture == ev->error_code)
+	  return 0;
+
   /* does not return */
   PrintXErrorAndCoredump(d, ev, MyName);
   return 0;
@@ -952,78 +1018,155 @@ void init_display (void)
   }
   XSetErrorHandler (handle_error);
   _XA_WM_DEL_WIN = XInternAtom(theDisplay, "WM_DELETE_WINDOW", 0);
-  InitPictureCMap (theDisplay);
+  PictureInitCMap (theDisplay);
   FScreenInit(theDisplay);
   AllocColorset(0);
   FShapeInit(theDisplay);
+  FRenderInit(theDisplay);
+
   x_fd = XConnectionNumber (theDisplay);
   theScreen = DefaultScreen (theDisplay);
   theRoot = RootWindow (theDisplay, theScreen);
-#ifdef TEST_MONO
-  Pdepth = 2;
-#endif
 
   ConsoleDebug (X11, "screen width: %d\n", globals.screen_g.width);
   ConsoleDebug (X11, "screen height: %d\n", globals.screen_g.height);
 }
 
+void recreate_background(WinManager *man, Contexts i)
+{
+	int cset = man->colorsets[i];
+
+	if (cset < 0)
+	{
+		return;
+	}
+
+	if (man->pixmap[i] && man->pixmap[i] != ParentRelative)
+	{
+		XFreePixmap(theDisplay, man->pixmap[i]);
+	}
+	man->pixmap[i]= None;
+	if ((i == DEFAULT && Colorset[cset].pixmap) ||
+	    (CSET_IS_TRANSPARENT(cset) && !CSET_IS_TRANSPARENT_PR_TINT(cset)))
+	{
+		man->pixmap[i] =
+			CreateBackgroundPixmap(
+				theDisplay, man->theWindow,
+				man->geometry.width, man->geometry.height,
+				&Colorset[cset],
+				Pdepth, man->backContext[i], False);
+		XSetTile(theDisplay, man->backContext[i], man->pixmap[i]);
+		XSetFillStyle(theDisplay, man->backContext[i], FillTiled);
+		if (i == DEFAULT)
+		{
+			XSetWindowBackgroundPixmap(
+				theDisplay, man->theWindow, man->pixmap[i]);
+		}
+	}
+	else if (Colorset[cset].pixmap && !CSET_IS_TRANSPARENT_PR_TINT(cset))
+	{
+		man->pixmap[i] =
+			CreateBackgroundPixmap(
+				theDisplay, man->theWindow,
+				man->geometry.boxwidth, man->geometry.boxheight,
+				&Colorset[cset],
+				Pdepth, man->backContext[i], False);
+		XSetTile(theDisplay, man->backContext[i], man->pixmap[i]);
+		XSetFillStyle(theDisplay, man->backContext[i], FillTiled);
+	}
+	else
+	{
+		XSetTile(theDisplay, man->backContext[i], None);
+		XSetFillStyle(theDisplay, man->backContext[i], FillSolid);
+		if (i == DEFAULT)
+		{
+			XSetWindowBackground (
+				theDisplay, man->theWindow, man->backcolor[i]);
+		}
+	}
+}
+
 void change_colorset(int color)
 {
-  WinManager *man;
-  int i,j;
+	WinManager *man;
+	int i,j;
 
-  for (j = 0; j < globals.num_managers; j++) {
-    man = &globals.managers[j];
-    for ( i = 0; i < NUM_CONTEXTS; i++ ) {
-      if(man->colorsets[i] == color) {
-        man->backcolor[i] = Colorset[color].bg;
-        man->forecolor[i] = Colorset[color].fg;
-        man->shadowcolor[i] = Colorset[color].shadow;
-        man->hicolor[i] = Colorset[color].hilite;
-
-	if (!man->backContext[i] || !man->hiContext[i] ||
-	    !man->flatContext[i] || !man->reliefContext[i] ||
-	    !man->shadowContext[i])
+	for (j = 0; j < globals.num_managers; j++)
 	{
-	  /* colorset not properly defined yet, just skip it */
-	  continue;
+		man = &globals.managers[j];
+		for (i = 0; i < NUM_CONTEXTS; i++)
+		{
+			if(man->colorsets[i] != color &&
+			   !(man->colorsets[i] == -2 &&
+			    man->colorsets[DEFAULT] == color))
+			{
+				continue;
+			}
+			man->backcolor[i] = Colorset[color].bg;
+			man->forecolor[i] = Colorset[color].fg;
+			man->shadowcolor[i] = Colorset[color].shadow;
+			man->hicolor[i] = Colorset[color].hilite;
+			if (!man->hiContext[i] ||
+			    !man->flatContext[i] ||
+			    !man->reliefContext[i] ||
+			    !man->shadowContext[i])
+			{
+				/* colorset not properly defined yet, skip it */
+				continue;
+			}
+			if (man->backContext[i])
+			{
+				XSetForeground(
+					theDisplay, man->backContext[i],
+					man->backcolor[i]);
+			}
+			XSetForeground(
+				theDisplay, man->hiContext[i],
+				man->forecolor[i]);
+			XSetBackground (
+				theDisplay, man->flatContext[i],
+				man->forecolor[i]);
+			XSetForeground (
+				theDisplay, man->flatContext[i],
+				man->backcolor[i]);
+			if (!PictureUseBWOnly()) {
+				XSetBackground (
+					theDisplay, man->reliefContext[i],
+					man->backcolor[i]);
+				XSetForeground (
+					theDisplay, man->reliefContext[i],
+					man->hicolor[i]);
+				XSetBackground (
+					theDisplay, man->shadowContext[i],
+					man->backcolor[i]);
+				XSetForeground (
+					theDisplay, man->shadowContext[i],
+					man->shadowcolor[i]);
+			}
+			recreate_background(man, i);
+		}
+		force_manager_redraw (man);
+	}
+}
+
+
+void recreate_transparent_bgs(WinManager *man)
+{
+	int i, color;
+
+	for (i = 0; i < NUM_CONTEXTS; i++ )
+	{
+		color = man->colorsets[i];
+		if (i == DEFAULT || !CSET_IS_TRANSPARENT(color) ||
+		    CSET_IS_TRANSPARENT_PR(color))
+		{
+			continue;
+		}
+		if (!man->backContext[i])
+		{
+			/* colorset not properly defined yet, just skip it */
+			continue;
+		}
+		recreate_background(man, i);
 	}
-	XSetForeground (theDisplay, man->backContext[i], man->backcolor[i]);
-	XSetForeground (theDisplay, man->hiContext[i], man->forecolor[i]);
-	XSetBackground (theDisplay, man->flatContext[i], man->forecolor[i]);
-	XSetForeground (theDisplay, man->flatContext[i], man->backcolor[i]);
-        if (Pdepth > 2) {
-	  XSetBackground (
-	    theDisplay, man->reliefContext[i], man->backcolor[i]);
-	  XSetForeground (
-	    theDisplay, man->reliefContext[i], man->hicolor[i]);
-	  XSetBackground (
-	    theDisplay, man->shadowContext[i], man->backcolor[i]);
-	  XSetForeground (
-	    theDisplay, man->shadowContext[i], man->shadowcolor[i]);
-        }
-
-        if (man->pixmap[i] && man->pixmap[i] != ParentRelative)
-          XFreePixmap(theDisplay, man->pixmap[i]);
-        if (Colorset[color].pixmap) {
-          man->pixmap[i] = CreateBackgroundPixmap(theDisplay, man->theWindow,
-			     man->geometry.width, man->geometry.height,
-			     &Colorset[color], Pdepth, man->backContext[i],
-			     False);
-          XSetTile(theDisplay, man->backContext[i], man->pixmap[i]);
-          XSetFillStyle(theDisplay, man->backContext[i], FillTiled);
-	  if (i == DEFAULT)
-	  {
-	    XSetWindowBackgroundPixmap(theDisplay, man->theWindow,
-				       man->pixmap[i]);
-	  }
-        } else {
-          man->pixmap[i] = None;
-          XSetFillStyle(theDisplay, man->backContext[i], FillSolid);
-        }
-
-        force_manager_redraw (man);
-      }
-    }
-  }
 }
