@@ -121,11 +121,11 @@ void EvtMouseCheckBox(struct XObj *xobj, XButtonEvent *EvtButton)
 
   while (End)
   {
-    XNextEvent(dpy, &event);
+    FNextEvent(dpy, &event);
     switch (event.type)
     {
     case EnterNotify:
-      XQueryPointer(dpy, *xobj->ParentWin,
+      FQueryPointer(dpy, *xobj->ParentWin,
 		    &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
       if (WinBut == 0)
       {
@@ -154,7 +154,7 @@ void EvtMouseCheckBox(struct XObj *xobj, XButtonEvent *EvtButton)
       }
       break;
     case LeaveNotify:
-      XQueryPointer(dpy, *xobj->ParentWin,
+      FQueryPointer(dpy, *xobj->ParentWin,
 		    &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
       if (Win2 == WinBut)
       {
