@@ -15,8 +15,8 @@
 
 #include "Tools.h"
 
-#define BdWidth 2		/* Border width */
-#define SbWidth 15		/* ScrollBar width */
+#define BdWidth 2               /* Border width */
+#define SbWidth 15              /* ScrollBar width */
 
 /*************************************************/
 /* Fonction pour Liste / Functions for the List  */
@@ -25,15 +25,9 @@ void InitList(struct XObj *xobj)
 {
   unsigned long mask;
   XSetWindowAttributes Attr;
-  int asc,desc,dir;
-  XCharStruct struc;
   int minw,minh,resize=0;
   int NbVisCell,NbCell;
-#ifdef I18N_MB
-  char **ml;
-  XFontStruct **fs_list;
-#endif
-  
+
   /* Enregistrement des couleurs et de la police / fonts and colors */
   if (xobj->colorset >= 0) {
     xobj->TabColor[fore] = Colorset[xobj->colorset].fg;
@@ -46,7 +40,7 @@ void InitList(struct XObj *xobj)
     xobj->TabColor[hili] = GetColor(xobj->hilicolor);
     xobj->TabColor[shad] = GetColor(xobj->shadcolor);
   }
-  
+
   mask = 0;
   Attr.background_pixel = xobj->TabColor[back];
   mask |= CWBackPixel;
@@ -61,27 +55,19 @@ void InitList(struct XObj *xobj)
   XSetForeground(dpy, xobj->gc, xobj->TabColor[fore]);
   XSetLineAttributes(dpy, xobj->gc, 1, LineSolid, CapRound, JoinMiter);
 
-#ifdef I18N_MB
-  if ((xobj->xfontset = GetFontSetOrFixed(dpy, xobj->font)) == NULL) {
-    fprintf(stderr, "FvwmScript: Couldn't load font. Exiting!\n");
+  if ((xobj->Ffont = FlocaleLoadFont(dpy, xobj->font, ScriptName)) == NULL)
+  {
+    fprintf(stderr, "%s: Couldn't load font. Exiting!\n", ScriptName);
     exit(1);
   }
-  XFontsOfFontSet(xobj->xfontset, &fs_list, &ml);
-  xobj->xfont = fs_list[0];
-#else
-  if ((xobj->xfont = GetFontOrFixed(dpy,xobj->font)) == NULL) {
-    fprintf(stderr, "FvwmScript: Couldn't load font. Exiting!\n");
-    exit(1);
-  }
-#endif
-  XSetFont(dpy, xobj->gc, xobj->xfont->fid);
+  if (xobj->Ffont->font != NULL)
+    XSetFont(dpy, xobj->gc, xobj->Ffont->font->fid);
 
   /* Calcul de la taille du widget *
    * Taille minimum: une ligne ou ascenseur visible */
   /* Computation of the size of the widget *
    * min size: one line or a visible elevator */
-  XTextExtents(xobj->xfont, "lp", strlen("lp"), &dir, &asc, &desc, &struc);
-  minh = 8 + 3*BdWidth + 3*(asc+desc+3);
+  minh = 8 + 3*BdWidth + 3*(xobj->Ffont->height + 3);
   if (xobj->height<minh)
   {
     xobj->height = minh;
@@ -102,7 +88,7 @@ void InitList(struct XObj *xobj)
 
   /* Calcul de la premiere cellule visible */
   /* Computation of the first visible cell */
-  NbVisCell = (xobj->height - 2 - 3*BdWidth) / (asc+desc+3);
+  NbVisCell = (xobj->height - 2 - 3*BdWidth) / (xobj->Ffont->height + 3);
   NbCell = CountOption(xobj->title);
   if (NbCell > NbVisCell)
   {
@@ -126,7 +112,7 @@ void DrawVSbList(struct XObj *xobj, int NbCell, int NbVisCell, int press)
   r.height = xobj->height - r.y - 2*BdWidth;
   r.width = SbWidth + 4;
   DrawReliefRect(r.x, r.y, r.width, r.height, xobj, shad, hili);
-  
+
   /* Calcul du rectangle pour les fleches *
    * Compute the rectangle for the arrows */
   r.x = r.x + 2;
@@ -173,28 +159,28 @@ void DrawCellule(struct XObj *xobj, int NbCell, int NbVisCell, int HeightCell,
     Title = (char*)GetMenuTitle(xobj->title, i);
     if (strlen(Title)!=0)
     {
-      int x = GetXTextPosition(xobj, r.width, 
-			       XTextWidth(xobj->xfont,Title,strlen(Title)),
+      int x = GetXTextPosition(xobj, r.width,
+			       FlocaleTextWidth(xobj->Ffont,Title,strlen(Title)),
 			       5 + r.x, 0, 0);
       if (xobj->value == i)
       {
 	/* hili is better than shad.*/
 	XSetForeground(dpy, xobj->gc, xobj->TabColor[hili]);
-	XFillRectangle(dpy, xobj->win, xobj->gc, r.x+2,  
+	XFillRectangle(dpy, xobj->win, xobj->gc, r.x+2,
 		       r.y +(i - xobj->value2)*HeightCell + 2,
 		       r.width, HeightCell-2);
-	DrawString(dpy, xobj, xobj->win, x,
-		   (i - xobj->value2) * HeightCell + asc + 2 + r.y, Title,
-		   strlen(Title), fore, back, shad, !xobj->flags[1]);
+	MyDrawString(dpy, xobj, xobj->win, x,
+		     (i - xobj->value2) * HeightCell + asc + 2 + r.y, Title,
+		     fore, back, shad, !xobj->flags[1]);
       }
       else
       {
 	XClearArea(dpy, xobj->win, r.x,
 		   r.y + (i-xobj->value2)*HeightCell + 2,
 		   r.width, HeightCell-2, False);
-	DrawString(dpy, xobj, xobj->win, x,
+	MyDrawString(dpy, xobj, xobj->win, x,
 		   (i - xobj->value2) * HeightCell + asc + 2 + r.y, Title,
-		   strlen(Title), fore, hili, back, !xobj->flags[1]);
+		   fore, hili, back, !xobj->flags[1]);
       }
     }
     else
@@ -216,11 +202,10 @@ void DestroyList(struct XObj *xobj)
 
 void DrawList(struct XObj *xobj)
 {
-  int asc,desc,dir,HeightCell;
-  XCharStruct struc;
   int NbVisCell,NbCell;
+  int HeightCell;
   XRectangle r;
-  
+
   /* Dessin du contour / */
   DrawReliefRect(0, 0, xobj->width, xobj->height, xobj, hili, shad);
 
@@ -232,8 +217,7 @@ void DrawList(struct XObj *xobj)
   DrawReliefRect(r.x, r.y, r.width, r.height, xobj, shad, hili);
 
   /* Calcul du nombre de cellules visibles */
-  XTextExtents(xobj->xfont, "lp", strlen("lp"), &dir, &asc, &desc, &struc);
-  HeightCell = asc+desc+3;
+  HeightCell = xobj->Ffont->height + 3;
   NbVisCell = r.height/HeightCell;
   NbCell = CountOption(xobj->title);
   if (NbCell > NbVisCell)
@@ -248,7 +232,7 @@ void DrawList(struct XObj *xobj)
 
 
   /* Dessin des cellules */
-  DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+  DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
 
   /* Dessin de l'ascenseur vertical */
   DrawVSbList(xobj, NbCell, NbVisCell, 0);
@@ -259,21 +243,18 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
 {
   XRectangle rect,rectT;
   XPoint pt;
-  XCharStruct struc;
   int x1,y1,x2,y2;
   Window Win1,Win2;
   unsigned int modif;
   int In = 1;
   static XEvent event;
-  int asc,desc,dir;
   int NbVisCell,NbCell,HeightCell,NPosCell,PosMouse;
   fd_set in_fdset;
- 
+
   pt.x = EvtButton->x-xobj->x;
   pt.y = EvtButton->y-xobj->y;
 
-  XTextExtents(xobj->xfont, "lp", strlen("lp"), &dir, &asc, &desc, &struc);
-  HeightCell = asc + desc + 3;
+  HeightCell = xobj->Ffont->height + 3;
   NbVisCell = (xobj->height-6-BdWidth)/HeightCell;
   NbCell = CountOption(xobj->title);
 
@@ -282,12 +263,11 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
   rect.y = rect.x;
   rect.width = xobj->width - rect.x - 10 - 2*BdWidth - SbWidth;
   rect.height = xobj->height -rect.y - 4 - 2*BdWidth;
-
   if(PtInRect(pt,rect))
   {
     /* Determination de la cellule */
     pt.y = pt.y - rect.y;
-    NPosCell = xobj->value2 + (pt.y/(asc+desc+3));
+    NPosCell = xobj->value2 + (pt.y/(xobj->Ffont->height+3));
     if (NPosCell > CountOption(xobj->title))
       NPosCell = 0;
     else if (NPosCell >= xobj->value2 + NbVisCell)
@@ -313,7 +293,7 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
     DrawVSbList(xobj, NbCell, NbVisCell, 1);
     do
     {
-      XQueryPointer(dpy, *xobj->ParentWin, &Win1, &Win2,
+      FQueryPointer(dpy, *xobj->ParentWin, &Win1, &Win2,
 		    &x1, &y1, &x2, &y2, &modif);
       pt.y = y2- xobj->y;
       pt.x = x2- xobj->x;
@@ -327,7 +307,8 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
 	    xobj->value2 = 1;
 	  else
 	  {
-	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell,
+			xobj->Ffont->ascent);
 	    DrawVSbList(xobj, NbCell, NbVisCell, 1);
 	  }
 	}
@@ -339,7 +320,8 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
 	  if (xobj->value2 < 1)
 	    xobj->value2 = 1;
 	  else
-	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell,
+			xobj->Ffont->ascent);
 	}
       }
       else
@@ -354,7 +336,7 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
       FD_SET(x_fd, &in_fdset);
       select(32, SELECT_FD_SET_CAST &in_fdset, NULL, NULL, NULL);
     }
-    while (!XCheckTypedEvent(dpy, ButtonRelease, &event));
+    while (!FCheckTypedEvent(dpy, ButtonRelease, &event));
     DrawVSbList(xobj, NbCell, NbVisCell, 0);
     return;
   }
@@ -366,7 +348,7 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
     DrawVSbList(xobj, NbCell, NbVisCell, 2);
     do
     {
-      XQueryPointer(dpy, *xobj->ParentWin, &Win1, &Win2,
+      FQueryPointer(dpy, *xobj->ParentWin, &Win1, &Win2,
 		    &x1, &y1, &x2, &y2, &modif);
       pt.y = y2 - xobj->y;
       pt.x = x2 - xobj->x;
@@ -378,7 +360,8 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
 	  if (xobj->value2 <= NbCell-NbVisCell)
 	  {
 	    xobj->value2++;
-	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell,
+			xobj->Ffont->ascent);
 	    DrawVSbList(xobj, NbCell, NbVisCell, 2);
 	  }
 	}
@@ -389,7 +372,8 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
 	  if (xobj->value2 <= NbCell-NbVisCell)
 	  {
 	    xobj->value2++;
-	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+	    DrawCellule(xobj, NbCell, NbVisCell, HeightCell,
+			xobj->Ffont->ascent);
 	  }
 	}
       }
@@ -405,7 +389,7 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
       FD_SET(x_fd, &in_fdset);
       select(32, SELECT_FD_SET_CAST &in_fdset, NULL, NULL, NULL);
     }
-    while (!XCheckTypedEvent(dpy, ButtonRelease, &event));
+    while (!FCheckTypedEvent(dpy, ButtonRelease, &event));
     DrawVSbList(xobj, NbCell, NbVisCell, 0);
     return;
   }
@@ -422,15 +406,13 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
     rectT.y = rect.y + (xobj->value2 - 1) * (rect.height - 8) / NbCell;
     if (NbVisCell<NbCell)
       rectT.height = NbVisCell * (rect.height-8) / NbCell+8;
-    else
-      rectT.height = rect.height;
     rectT.width = rect.width;
     if(PtInRect(pt,rectT))
     {
       PosMouse = pt.y - rectT.y - HeightCell / 2 + 2;
       do
       {
-	XQueryPointer(dpy, *xobj->ParentWin, &Win1, &Win2,
+	FQueryPointer(dpy, *xobj->ParentWin, &Win1, &Win2,
 		      &x1, &y1, &x2, &y2, &modif);
 	/* Calcul de l'id de la premiere cellule */
 	pt.y = y2-xobj->y - PosMouse;
@@ -447,14 +429,14 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
 	if (xobj->value2 != NPosCell)
 	{
 	  xobj->value2 = NPosCell;
-	  DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+	  DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
 	  DrawVSbList(xobj, NbCell, NbVisCell, 0);
 	}
 	FD_ZERO(&in_fdset);
 	FD_SET(x_fd, &in_fdset);
 	select(32, SELECT_FD_SET_CAST &in_fdset, NULL, NULL, NULL);
       }
-      while (!XCheckTypedEvent(dpy, ButtonRelease, &event));
+      while (!FCheckTypedEvent(dpy, ButtonRelease, &event));
     }
     else if (pt.y < rectT.y)
     {
@@ -464,7 +446,7 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
       if (xobj->value2 != NPosCell)
       {
 	xobj->value2 = NPosCell;
-	DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+	DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
 	DrawVSbList(xobj, NbCell, NbVisCell, 0);
       }
     }
@@ -481,7 +463,7 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
       if (xobj->value2 != NPosCell)
       {
 	xobj->value2 = NPosCell;
-	DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+	DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
 	DrawVSbList(xobj, NbCell, NbVisCell, 0);
       }
     }
@@ -489,11 +471,9 @@ void EvtMouseList(struct XObj *xobj, XButtonEvent *EvtButton)
 }
 
 void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
-{ 
+{
   KeySym ks;
   unsigned char buf[10];
-  XCharStruct struc;
-  int asc,desc,dir;
   int NbVisCell,NbCell,HeightCell,NPosCell;
   Window Win1,Win2;
   int x1,y1,x2,y2;
@@ -501,11 +481,10 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
   XRectangle rect;
   XPoint pt;
 
-  XTextExtents(xobj->xfont, "lp", strlen("lp"), &dir, &asc, &desc, &struc);
-  HeightCell = asc + desc + 3;
+  HeightCell = xobj->Ffont->height + 3;
   NbVisCell = (xobj->height - 6 - BdWidth) / HeightCell;
   NbCell = CountOption(xobj->title);
-  XQueryPointer(dpy, xobj->win, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
+  FQueryPointer(dpy, xobj->win, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
   pt.x = x2;
   pt.y = y2;
   rect.x = 4 + BdWidth;
@@ -522,7 +501,7 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
     {
       /* Determination de la cellule */
       pt.y =  pt.y - rect.y;
-      NPosCell = xobj->value2 + (pt.y/(asc+desc+3));
+      NPosCell = xobj->value2 + (pt.y/(xobj->Ffont->height + 3));
       if (NPosCell > CountOption(xobj->title))
 	NPosCell = 0;
       else if (NPosCell >= xobj->value2 + NbVisCell)
@@ -545,19 +524,19 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
       {
 	/* Determination de la cellule */
 	pt.y= pt.y - rect.y;
-	NPosCell = xobj->value2+(pt.y/(asc+desc+3));
+	NPosCell = xobj->value2+(pt.y/(xobj->Ffont->height + 3));
 	if (NPosCell > CountOption(xobj->title))
 	  NPosCell = 0;
 	if (NPosCell > 1)
 	{
-	  XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, -HeightCell);
+	  FWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, -HeightCell);
 	}
       }
     }
     else
     {
       xobj->value2--;
-      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
       DrawVSbList(xobj, NbCell, NbVisCell, 0);
     }
   }
@@ -569,19 +548,19 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
       {
 	/* Determination de la cellule */
 	pt.y =  pt.y - rect.y;
-	NPosCell =  xobj->value2 + (pt.y/(asc+desc+3));
+	NPosCell =  xobj->value2 + (pt.y/(xobj->Ffont->height + 3));
 	if (NPosCell > CountOption(xobj->title))
 	  NPosCell = 0;
 	if (NPosCell < NbCell && NPosCell > 0)
 	{
-	  XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, HeightCell);
+	  FWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, HeightCell);
 	}
       }
     }
     else
     {
       xobj->value2++;
-      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
       DrawVSbList(xobj, NbCell, NbVisCell, 0);
     }
   }
@@ -590,7 +569,7 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
     xobj->value2 = xobj->value2 - NbVisCell;
     if (xobj->value2 < 1)
       xobj->value2 = 1;
-    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
     DrawVSbList(xobj, NbCell, NbVisCell, 0);
   }
   else if (ks == XK_Next && xobj->value2 <= NbCell-NbVisCell)
@@ -598,7 +577,7 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
     xobj->value2 = xobj->value2 + NbVisCell;
     if (xobj->value2 > NbCell - NbVisCell + 1)
       xobj->value2 = NbCell - NbVisCell + 1;
-    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+    DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
     DrawVSbList(xobj, NbCell, NbVisCell, 0);
   }
   else if (ks == XK_Home)
@@ -606,7 +585,7 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
     if (xobj->value2 > 1)
     {
       xobj->value2 = 1;
-      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
       DrawVSbList(xobj, NbCell, NbVisCell, 0);
     }
   }
@@ -615,7 +594,7 @@ void EvtKeyList(struct XObj *xobj, XKeyEvent *EvtKey)
     if (xobj->value2 < NbCell - NbVisCell + 1)
     {
       xobj->value2 = NbCell - NbVisCell + 1;
-      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, asc);
+      DrawCellule(xobj, NbCell, NbVisCell, HeightCell, xobj->Ffont->ascent);
       DrawVSbList(xobj, NbCell, NbVisCell, 0);
     }
   }
