@@ -211,7 +211,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
 
   while (End)
   {
-    XQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
+    FQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
     /* Determiner l'option courante / Current option */
     y2 = y2 - y;
     x2 = x2 - x;
@@ -228,7 +228,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
       oldvalue = newvalue;
     }
 
-    XNextEvent(dpy, &event);
+    FNextEvent(dpy, &event);
     switch (event.type)
     {
     case KeyPress:
@@ -241,10 +241,10 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
 	End = 0;
       }
       else if (ks == XK_Up && y2 >= hOpt) {
-	XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, -hOpt);
+	FWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, -hOpt);
       }
       else if (ks == XK_Down && y2 + hOpt <= hMenu) {
-	XWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, hOpt);
+	FWarpPointer(dpy, None, None, 0, 0, 0, 0, 0, hOpt);
       }
       break;
     case ButtonPress:
@@ -264,7 +264,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
 #if 0
   do
   {
-    XQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
+    FQueryPointer(dpy, Root, &Win1, &Win2, &x1, &y1, &x2, &y2, &modif);
     /* Determiner l'option courante / Current option */
     y2 = y2 - y;
     x2 = x2 - x;
@@ -286,7 +286,7 @@ void EvtMousePopupMenu(struct XObj *xobj, XButtonEvent *EvtButton)
     FD_SET(x_fd, &in_fdset);
     select(32, SELECT_FD_SET_CAST &in_fdset, NULL, NULL, NULL);
   }
-  while (!XCheckTypedEvent(dpy, while_mask, &event));
+  while (!FCheckTypedEvent(dpy, while_mask, &event));
 #endif
  XUngrabPointer(dpy, CurrentTime);
  XSync(dpy,0);
