@@ -35,12 +35,12 @@
 
 typedef struct {
   int  x, y, w, h, tw, th, open, type;
-  int px, py;
   char *text;
   Window win;
 } TipStruct;
 
 Bool GoodiesParseConfig(char *tline);
+void LoadGoodiesFont(void);
 void InitGoodies(void);
 void DrawGoodies(void);
 int MouseInClock(int x, int y);
